@@ -18,6 +18,7 @@
 /*****************************************************************************/
 
 #include "gftp.h"
+static const char cvsid[] = "$Id: config_file.c,v 1.10 2002/11/21 02:49:06 masneyb Exp $";
 
 static void write_comment 			( FILE * fd, 
 						  const char *comment );
@@ -29,174 +30,174 @@ static int parse_args 				( char *str,
 
 gftp_config_vars config_file_vars[] = 
 {
-  {"", N_("General"), (void *) 0x1, CONFIG_NOTEBOOK, 1, ""},
-  {"", "", (void *) 0x1, CONFIG_TABLE, 1, ""},
-  {"email", N_("Email address:"), &emailaddr, CONFIG_CHARTEXT, 1, 
-	N_("Enter your email address here")},
-  {"view_program", N_("View program:"), &view_program, CONFIG_CHARTEXT, 1,
-	N_("The default program used to view files. If this is blank, the internal file viewer will be used")},
-  {"edit_program", N_("Edit program:"), &edit_program, CONFIG_CHARTEXT, 1,
-	N_("The default program used to edit files.")},
-  {"startup_directory", N_("Startup Directory:"), &startup_directory,
-        CONFIG_CHARTEXT, 1,
-        N_("The default directory gFTP will go to on startup")},
+  {"", N_("General"), (void *) 0x1, CONFIG_NOTEBOOK, "", NULL, GFTP_PORT_GTK},
+  {"", "", (void *) 0x1, CONFIG_TABLE, "", NULL, GFTP_PORT_GTK},
+  {"email", N_("Email address:"), &emailaddr, CONFIG_CHARTEXT, 
+	N_("Enter your email address here"), NULL, GFTP_PORT_ALL},
+  {"view_program", N_("View program:"), &view_program, CONFIG_CHARTEXT,
+	N_("The default program used to view files. If this is blank, the internal file viewer will be used"), NULL, GFTP_PORT_ALL},
+  {"edit_program", N_("Edit program:"), &edit_program, CONFIG_CHARTEXT,
+	N_("The default program used to edit files."), NULL, GFTP_PORT_GTK},
+  {"startup_directory", N_("Startup Directory:"), &startup_directory, CONFIG_CHARTEXT,
+        N_("The default directory gFTP will go to on startup"), NULL, GFTP_PORT_ALL},
   {"max_log_window_size", N_("Max Log Window Size:"), &max_log_window_size, CONFIG_UINTTEXT,
-        1, N_("The maximum size of the log window in bytes for the GTK+ port")},
-  {"", "", (void *) 0x1, CONFIG_TABLE, 1, ""},
+        N_("The maximum size of the log window in bytes for the GTK+ port"), NULL, GFTP_PORT_GTK},
+  {"", "", (void *) 0x1, CONFIG_TABLE, "", NULL, GFTP_PORT_GTK},
   {"append_transfers", N_("Append file transfers"), &append_file_transfers,
-        CONFIG_CHECKBOX, 1,
-        N_("Append new file transfers onto existing ones")},
-  {"one_transfer", N_("Do one transfer at a time"),  &do_one_transfer_at_a_time, CONFIG_CHECKBOX, 1, N_("Do only one transfer at a time?")}, 
-  {"overwrite_default", N_("Overwrite by Default"), &overwrite_by_default,
-        CONFIG_CHECKBOX, 1,
-        N_("Overwrite files by default or set to resume file transfers")},
-  {"refresh_files", N_("Refresh after each file transfer"), &refresh_files,
-	CONFIG_CHECKBOX, 1,
-	N_("Refresh the listbox after each file is transfered")},
-  {"sort_dirs_first", N_("Sort directories first"), &sort_dirs_first, 
-	CONFIG_CHECKBOX, 1, N_("Put the directories first then the files")},
-  {"start_transfers", N_("Start file transfers"), &start_file_transfers, 
-	CONFIG_CHECKBOX, 1,
-	N_("Automatically start the file transfers when they get queued?")},
-  {"show_hidden_files", N_("Show hidden files"), &show_hidden_files, 
-	CONFIG_CHECKBOX, 1, N_("Show hidden files in the listboxes")},
+        CONFIG_CHECKBOX,
+        N_("Append new file transfers onto existing ones"), NULL, GFTP_PORT_GTK},
+  {"one_transfer", N_("Do one transfer at a time"),  &do_one_transfer_at_a_time, CONFIG_CHECKBOX, 
+	N_("Do only one transfer at a time?"), NULL, GFTP_PORT_GTK}, 
+  {"overwrite_default", N_("Overwrite by Default"), &overwrite_by_default, CONFIG_CHECKBOX,
+        N_("Overwrite files by default or set to resume file transfers"), NULL, GFTP_PORT_GTK},
+  {"refresh_files", N_("Refresh after each file transfer"), &refresh_files, CONFIG_CHECKBOX, 
+	N_("Refresh the listbox after each file is transfered"), NULL, GFTP_PORT_GTK},
+  {"sort_dirs_first", N_("Sort directories first"), &sort_dirs_first, CONFIG_CHECKBOX, 
+	N_("Put the directories first then the files"), NULL, GFTP_PORT_ALL},
+  {"start_transfers", N_("Start file transfers"), &start_file_transfers, CONFIG_CHECKBOX, 
+	N_("Automatically start the file transfers when they get queued?"), NULL, GFTP_PORT_GTK},
+  {"show_hidden_files", N_("Show hidden files"), &show_hidden_files, CONFIG_CHECKBOX, 
+	N_("Show hidden files in the listboxes"), NULL, GFTP_PORT_ALL},
   
-  {"", N_("Network"), (void *) 0x1, CONFIG_NOTEBOOK, 1, ""},
-  {"", "", (void *) 0x1, CONFIG_TABLE, 1, ""},
+  {"", N_("Network"), (void *) 0x1, CONFIG_NOTEBOOK, "", NULL, GFTP_PORT_GTK},
+  {"", "", (void *) 0x1, CONFIG_TABLE, "", NULL, GFTP_PORT_GTK},
   {"network_timeout", N_("Network timeout:"), &network_timeout, CONFIG_INTTEXT, 
-	1, N_("The timeout waiting for network input/output. This is NOT an idle timeout.")},
-  {"retries", N_("Connect retries:"), &retries, CONFIG_INTTEXT, 1,
-	N_("The number of auto-retries to do. Set this to 0 to retry indefinately")},
-  {"sleep_time", N_("Retry sleep time:"), &sleep_time, CONFIG_INTTEXT, 1,
-	N_("The number of seconds to wait between retries")},
-  {"maxkbs", N_("Max KB/S:"), &maxkbs, CONFIG_FLOATTEXT, 1,
-	N_("The maximum KB/s a file transfer can get. (Set to 0 to disable)")},
-  {"", N_("Default Protocol"), (void *) 0x1, CONFIG_COMBO, 1, "DP"},
+	N_("The timeout waiting for network input/output. This is NOT an idle timeout."), NULL, GFTP_PORT_ALL},
+  {"retries", N_("Connect retries:"), &retries, CONFIG_INTTEXT, 
+	N_("The number of auto-retries to do. Set this to 0 to retry indefinately"), NULL, GFTP_PORT_ALL},
+  {"sleep_time", N_("Retry sleep time:"), &sleep_time, CONFIG_INTTEXT, 
+	N_("The number of seconds to wait between retries"), NULL, GFTP_PORT_ALL},
+  {"maxkbs", N_("Max KB/S:"), &maxkbs, CONFIG_FLOATTEXT, 
+	N_("The maximum KB/s a file transfer can get. (Set to 0 to disable)"), NULL, GFTP_PORT_ALL},
+  {"", N_("Default Protocol"), (void *) 0x1, CONFIG_COMBO, "DP", NULL, GFTP_PORT_GTK},
   {"default_protocol", N_("Default Protocol"), &default_protocol, 
-        CONFIG_CHARTEXT, 0, N_("This specifies the default protocol to use")},
+        CONFIG_CHARTEXT, N_("This specifies the default protocol to use"), NULL, 0},
 
-  {"", N_("FTP"), (void *) 0x1, CONFIG_NOTEBOOK, 1, ""},
-  {"", "", (void *) 0x1, CONFIG_TABLE, 1, ""},
-  {"firewall_host", N_("Proxy hostname:"), &firewall_host, CONFIG_CHARTEXT, 1,
-	N_("Firewall hostname")},
-  {"firewall_port", N_("Proxy port:"), &firewall_port, CONFIG_INTTEXT, 1,
-	N_("Port to connect to on the firewall")},
+  {"", N_("FTP"), (void *) 0x1, CONFIG_NOTEBOOK, "", NULL, GFTP_PORT_GTK},
+  {"", "", (void *) 0x1, CONFIG_TABLE, "", NULL, GFTP_PORT_GTK},
+  {"firewall_host", N_("Proxy hostname:"), &firewall_host, CONFIG_CHARTEXT,
+	N_("Firewall hostname"), NULL, GFTP_PORT_ALL},
+  {"firewall_port", N_("Proxy port:"), &firewall_port, CONFIG_INTTEXT,
+	N_("Port to connect to on the firewall"), NULL, GFTP_PORT_ALL},
   {"firewall_username", N_("Proxy username:"), &firewall_username,
-	CONFIG_CHARTEXT, 1, N_("Your firewall username")},
+	CONFIG_CHARTEXT, N_("Your firewall username"), NULL, GFTP_PORT_ALL},
   {"firewall_password", N_("Proxy password:"), &firewall_password, 
-	CONFIG_CHARPASS, 1, N_("Your firewall password")},
-  {"firewall_account", N_("Proxy account:"), &firewall_account, CONFIG_CHARTEXT, 1, N_("Your firewall account (optional)")},
-  {"", "", (void *) 0x1, CONFIG_TABLE, 1, ""},
+	CONFIG_CHARPASS, N_("Your firewall password"), NULL, GFTP_PORT_ALL},
+  {"firewall_account", N_("Proxy account:"), &firewall_account, CONFIG_CHARTEXT, 
+	N_("Your firewall account (optional)"), NULL, GFTP_PORT_ALL},
+  {"", "", (void *) 0x1, CONFIG_TABLE, "", NULL, GFTP_PORT_GTK},
 
-  {"passive_transfer", N_("Passive file transfers"), &passive_transfer, 
-	CONFIG_CHECKBOX, 1,
-	N_("Send PASV command or PORT command for data transfers")},
-  {"resolve_symlinks", N_("Resolve Remote Symlinks (LIST -L)"), &resolve_symlinks, 
-	CONFIG_CHECKBOX, 1,
-	N_("If you disable this feature, then gFTP will only send LIST to the remote server instead of LIST -L")},
+  {"passive_transfer", N_("Passive file transfers"), &passive_transfer, CONFIG_CHECKBOX, 
+	N_("Send PASV command or PORT command for data transfers"), NULL, GFTP_PORT_ALL},
+  {"resolve_symlinks", N_("Resolve Remote Symlinks (LIST -L)"), &resolve_symlinks, CONFIG_CHECKBOX, 
+	N_("If you disable this feature, then gFTP will only send LIST to the remote server instead of LIST -L"), NULL, GFTP_PORT_ALL},
 
-  {"", "", (void *) 0x1, CONFIG_TABLE, 1, ""},
-  {"", N_("Proxy server type"), (void *) 0x1, CONFIG_COMBO, 1, "PS"},
-  {"proxy_config", N_("Proxy config"), &proxy_config, CONFIG_TEXT, 1,
-	N_("This specifies how your proxy server expects us to log in")},
-  {"", N_("%pu = proxy user"), (void *) 0x1, CONFIG_LABEL, 1, ""},
-  {"", N_("%hu = host user"), (void *) 0x1, CONFIG_LABEL, 1, ""},
-  {"", N_("%pp = proxy pass"), (void *) 0x1, CONFIG_LABEL, 1, ""},
-  {"", N_("%hp = host pass"), (void *) 0x1, CONFIG_LABEL, 1, ""},
-  {"", N_("%ph = proxy host"), (void *) 0x1, CONFIG_LABEL, 1, ""},
-  {"", N_("%hh = host"), (void *) 0x1, CONFIG_LABEL, 1, ""},
-  {"", N_("%po = proxy port"), (void *) 0x1, CONFIG_LABEL, 1, ""},
-  {"", N_("%ho = host port"), (void *) 0x1, CONFIG_LABEL, 1, ""},
-  {"", N_("%pa = proxy account"), (void *) 0x1, CONFIG_LABEL, 1, ""},
-  {"", N_("%ha = host account"), (void *) 0x1, CONFIG_LABEL, 1, ""},
+  {"", "", (void *) 0x1, CONFIG_TABLE, "", NULL, GFTP_PORT_GTK},
+  {"", N_("Proxy server type"), (void *) 0x1, CONFIG_COMBO, "PS", NULL, GFTP_PORT_GTK},
+  {"proxy_config", N_("Proxy config"), &proxy_config, CONFIG_TEXT,
+	N_("This specifies how your proxy server expects us to log in"), NULL, GFTP_PORT_GTK},
+  {"", N_("%pu = proxy user"), (void *) 0x1, CONFIG_LABEL, "", NULL, 0},
+  {"", N_("%hu = host user"), (void *) 0x1, CONFIG_LABEL, "", NULL, 0},
+  {"", N_("%pp = proxy pass"), (void *) 0x1, CONFIG_LABEL, "", NULL, 0},
+  {"", N_("%hp = host pass"), (void *) 0x1, CONFIG_LABEL, "", NULL, 0},
+  {"", N_("%ph = proxy host"), (void *) 0x1, CONFIG_LABEL, "", NULL, 0},
+  {"", N_("%hh = host"), (void *) 0x1, CONFIG_LABEL, "", NULL, 0},
+  {"", N_("%po = proxy port"), (void *) 0x1, CONFIG_LABEL, "", NULL, 0},
+  {"", N_("%ho = host port"), (void *) 0x1, CONFIG_LABEL, "", NULL, 0},
+  {"", N_("%pa = proxy account"), (void *) 0x1, CONFIG_LABEL, "", NULL, 0},
+  {"", N_("%ha = host account"), (void *) 0x1, CONFIG_LABEL, "", NULL, 0},
 
-  {"", N_("HTTP"), (void *) 0x1, CONFIG_NOTEBOOK, 1, ""},
-  {"", "", (void *) 0x1, CONFIG_TABLE, 1, ""},
+  {"", N_("HTTP"), (void *) 0x1, CONFIG_NOTEBOOK, "", NULL, GFTP_PORT_GTK},
+  {"", "", (void *) 0x1, CONFIG_TABLE, "", NULL, GFTP_PORT_GTK},
   {"http_proxy_host", N_("Proxy hostname:"), &http_proxy_host, CONFIG_CHARTEXT,
-	1, N_("Firewall hostname")},
-  {"http_proxy_port", N_("Proxy port:"), &http_proxy_port, CONFIG_INTTEXT, 1,
-	N_("Port to connect to on the firewall")},
+	N_("Firewall hostname"), NULL, GFTP_PORT_ALL},
+  {"http_proxy_port", N_("Proxy port:"), &http_proxy_port, CONFIG_INTTEXT, 
+	N_("Port to connect to on the firewall"), NULL, GFTP_PORT_ALL},
   {"http_proxy_username", N_("Proxy username:"), &http_proxy_username, 
-	CONFIG_CHARTEXT, 1, N_("Your firewall username")},
+	CONFIG_CHARTEXT, N_("Your firewall username"), NULL, GFTP_PORT_ALL},
   {"http_proxy_password", N_("Proxy password:"), &http_proxy_password, 
-	CONFIG_CHARPASS, 1, N_("Your firewall password")},
-  {"", "", (void *) 0x1, CONFIG_TABLE, 1, ""},
-  {"use_http11", N_("Use HTTP/1.1"), &use_http11, CONFIG_CHECKBOX, 1,
-	N_("Do you want to use HTTP/1.1 or HTTP/1.0")},
+	CONFIG_CHARPASS, N_("Your firewall password"), NULL, GFTP_PORT_ALL},
+  {"", "", (void *) 0x1, CONFIG_TABLE, "", NULL, GFTP_PORT_GTK},
+  {"use_http11", N_("Use HTTP/1.1"), &use_http11, CONFIG_CHECKBOX,
+	N_("Do you want to use HTTP/1.1 or HTTP/1.0"), NULL, GFTP_PORT_ALL},
 
-  {"", N_("SSH"), (void *) 0x1, CONFIG_NOTEBOOK, 1, ""},
-  {"", "", (void *) 0x1, CONFIG_TABLE, 1, ""},
-  {"ssh_prog_name", N_("SSH Prog Name:"), &ssh_prog_name, CONFIG_CHARTEXT, 1,
-	N_("The path to the SSH executable")},
+  {"", N_("SSH"), (void *) 0x1, CONFIG_NOTEBOOK, "", NULL, GFTP_PORT_GTK},
+  {"", "", (void *) 0x1, CONFIG_TABLE, "", NULL, GFTP_PORT_GTK},
+  {"ssh_prog_name", N_("SSH Prog Name:"), &ssh_prog_name, CONFIG_CHARTEXT,
+	N_("The path to the SSH executable"), NULL, GFTP_PORT_ALL},
   {"ssh_extra_params", N_("SSH Extra Params:"), &ssh_extra_params, 
-        CONFIG_CHARTEXT, 1, N_("Extra parameters to pass to the SSH program")},
+        CONFIG_CHARTEXT, N_("Extra parameters to pass to the SSH program"), NULL, GFTP_PORT_ALL},
   {"ssh1_sftp_path", N_("SSH sftpserv path:"), &ssh1_sftp_path,
-	CONFIG_CHARTEXT, 1, N_("Default remote SSH sftpserv path")},
+	CONFIG_CHARTEXT, N_("Default remote SSH sftpserv path"), NULL, GFTP_PORT_ALL},
   {"ssh2_sftp_path", N_("SSH2 sftp-server path:"), &ssh2_sftp_path,
-	CONFIG_CHARTEXT, 1, N_("Default remote SSH2 sftp-server path")},
-  {"", "", (void *) 0x1, CONFIG_TABLE, 1, ""},
-  {"ssh_need_userpass", N_("Need SSH User/Pass"), &ssh_need_userpass,
-        CONFIG_CHECKBOX, 1,
-        N_("Require a username/password for SSH connections")},
-  {"ssh_use_askpass", N_("Use ssh-askpass util"), &ssh_use_askpass,
-        CONFIG_CHECKBOX, 1,
-        N_("Use the ssh-askpass utility to grab the users password")},
-  {"sshv2_use_sftp_subsys", N_("Use SSH2 SFTP subsys"), 
-        &sshv2_use_sftp_subsys, CONFIG_CHECKBOX, 1,
-        N_("Call ssh with the -s sftp flag. This is helpful because you won't have to know the remote path to the remote sftp-server")},
-  {"enable_old_ssh", N_("Enable old SSH protocol"), &enable_old_ssh,
-        CONFIG_CHECKBOX, 0,
-        N_("Enable the old SSH protocol. You will need to download the sftp server from http:///www.xbill.org/sftp")},
+	CONFIG_CHARTEXT, N_("Default remote SSH2 sftp-server path"), NULL, GFTP_PORT_ALL},
+  {"", "", (void *) 0x1, CONFIG_TABLE, "", NULL, GFTP_PORT_GTK},
+  {"ssh_need_userpass", N_("Need SSH User/Pass"), &ssh_need_userpass, CONFIG_CHECKBOX, 
+        N_("Require a username/password for SSH connections"), NULL, GFTP_PORT_ALL},
+  {"ssh_use_askpass", N_("Use ssh-askpass util"), &ssh_use_askpass, CONFIG_CHECKBOX, 
+        N_("Use the ssh-askpass utility to grab the users password"), NULL, GFTP_PORT_GTK},
+  {"sshv2_use_sftp_subsys", N_("Use SSH2 SFTP subsys"), &sshv2_use_sftp_subsys, CONFIG_CHECKBOX, 
+        N_("Call ssh with the -s sftp flag. This is helpful because you won't have to know the remote path to the remote sftp-server"), NULL, GFTP_PORT_GTK},
+  {"enable_old_ssh", N_("Enable old SSH protocol"), &enable_old_ssh, CONFIG_CHECKBOX, 
+        N_("Enable the old SSH protocol. You will need to download the sftp server from http:///www.xbill.org/sftp"), NULL, GFTP_PORT_ALL},
 
-  {"list_dblclk_action", "", &listbox_dblclick_action, CONFIG_HIDEINT, 0,
-	N_("This defines what will happen when you double click a file in the file listboxes. 0=View file 1=Edit file 2=Transfer file")},
-  {"use_default_dl_types", "", &use_default_dl_types, CONFIG_HIDEINT, 0,
-	N_("(*) If this is set, and there is a ext= line below for the file extension, it will download the file as specified below")},
-  {"listbox_local_width", "", &listbox_local_width, CONFIG_HIDEINT, 0,
-	N_("The default width of the local files listbox")},
-  {"listbox_remote_width", "", &listbox_remote_width, CONFIG_HIDEINT, 0,
-	N_("The default width of the remote files listbox")},
-  {"listbox_file_height", "", &listbox_file_height, CONFIG_HIDEINT, 0,
-	N_("The default height of the local/remote files listboxes")},
-  {"transfer_height", "", &transfer_height, CONFIG_HIDEINT, 0,
-	N_("The default height of the transfer listbox")},
-  {"log_height", "", &log_height, CONFIG_HIDEINT, 0,
-	N_("The default height of the logging window")},
-  {"file_trans_column", "", &file_trans_column, CONFIG_HIDEINT, 0,
-	N_("The width of the filename column in the transfer window. Set this to 0 to have this column automagically resize.")},
-  {"local_file_width", "", &local_columns[0], CONFIG_HIDEINT, 0,
-	N_("The width of the filename column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column")},
-  {"local_size_width", "", &local_columns[1], CONFIG_HIDEINT, 0,
-	N_("The width of the size column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column")},
-  {"local_user_width", "", &local_columns[2], CONFIG_HIDEINT, 0,
-	N_("The width of the user column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column")},
-  {"local_group_width", "", &local_columns[3], CONFIG_HIDEINT, 0,
-	N_("The width of the group column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column")},
-  {"local_date_width", "", &local_columns[4], CONFIG_HIDEINT, 0,
-	N_("The width of the date column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column")},
-  {"local_attribs_width", "", &local_columns[5], CONFIG_HIDEINT, 0,
-	N_("The width of the attribs column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column")},
-  {"remote_file_width", "", &remote_columns[0], CONFIG_HIDEINT, 0,
-	N_("The width of the filename column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column")},
-  {"remote_size_width", "", &remote_columns[1], CONFIG_HIDEINT, 0,
-	N_("The width of the size column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column")},
-  {"remote_user_width", "", &remote_columns[2], CONFIG_HIDEINT, 0,
-	N_("The width of the user column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column")},
-  {"remote_group_width", "", &remote_columns[3], CONFIG_HIDEINT, 0,
-	N_("The width of the group column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column")},
-  {"remote_date_width", "", &remote_columns[4], CONFIG_HIDEINT, 0,
-	N_("The width of the date column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column")},
-  {"remote_attribs_width", "", &remote_columns[5], CONFIG_HIDEINT, 0,
-	N_("The width of the attribs column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column")},
-  {"send_color", "", &send_color, CONFIG_COLOR, 0,
-        N_("The color of the commands that are sent to the server")},
-  {"recv_color", "", &recv_color, CONFIG_COLOR, 0,
-        N_("The color of the commands that are received from the server")},
-  {"error_color", "", &error_color, CONFIG_COLOR, 0,
-        N_("The color of the error messages")},
-  {"misc_color", "", &misc_color, CONFIG_COLOR, 0,
-        N_("The color of the rest of the log messages")},
-  {NULL, NULL, NULL, 0, 0, ""}
+  {"list_dblclk_action", "", &listbox_dblclick_action, CONFIG_HIDEINT, 
+	N_("This defines what will happen when you double click a file in the file listboxes. 0=View file 1=Edit file 2=Transfer file"), NULL, 0},
+  {"use_default_dl_types", "", &use_default_dl_types, CONFIG_HIDEINT, 
+	N_("(*) If this is set, and there is a ext= line below for the file extension, it will download the file as specified below"), NULL, 0},
+  {"listbox_local_width", "", &listbox_local_width, CONFIG_HIDEINT, 
+	N_("The default width of the local files listbox"), NULL, 0},
+  {"listbox_remote_width", "", &listbox_remote_width, CONFIG_HIDEINT,
+	N_("The default width of the remote files listbox"), NULL, 0},
+  {"listbox_file_height", "", &listbox_file_height, CONFIG_HIDEINT,
+	N_("The default height of the local/remote files listboxes"), NULL, 0},
+  {"transfer_height", "", &transfer_height, CONFIG_HIDEINT,
+	N_("The default height of the transfer listbox"), NULL, 0},
+  {"log_height", "", &log_height, CONFIG_HIDEINT,
+	N_("The default height of the logging window"), NULL, 0},
+  {"file_trans_column", "", &file_trans_column, CONFIG_HIDEINT,
+	N_("The width of the filename column in the transfer window. Set this to 0 to have this column automagically resize."), NULL, 0},
+  {"local_sortcol", "", &local_sortcol, CONFIG_INTTEXT, 
+	N_("The default column to sort by"), NULL, GFTP_PORT_TEXT},
+  {"local_sortasds", "", &local_sortasds, CONFIG_INTTEXT,
+	N_("Sort ascending or descending"), NULL, GFTP_PORT_TEXT},
+  {"remote_sortcol", "", &remote_sortcol, CONFIG_INTTEXT,
+	N_("The default column to sort by"), NULL, GFTP_PORT_TEXT},
+  {"remote_sortasds", "", &remote_sortasds, CONFIG_INTTEXT,
+	N_("Sort ascending or descending"), NULL, GFTP_PORT_TEXT},
+  {"local_file_width", "", &local_columns[0], CONFIG_HIDEINT,
+	N_("The width of the filename column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column"), NULL, 0},
+  {"local_size_width", "", &local_columns[1], CONFIG_HIDEINT,
+	N_("The width of the size column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column"), NULL, 0},
+  {"local_user_width", "", &local_columns[2], CONFIG_HIDEINT,
+	N_("The width of the user column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column"), NULL, 0},
+  {"local_group_width", "", &local_columns[3], CONFIG_HIDEINT,
+	N_("The width of the group column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column"), NULL, 0},
+  {"local_date_width", "", &local_columns[4], CONFIG_HIDEINT,
+	N_("The width of the date column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column"), NULL, 0},
+  {"local_attribs_width", "", &local_columns[5], CONFIG_HIDEINT,
+	N_("The width of the attribs column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column"), NULL, 0},
+  {"remote_file_width", "", &remote_columns[0], CONFIG_HIDEINT,
+	N_("The width of the filename column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column"), NULL, 0},
+  {"remote_size_width", "", &remote_columns[1], CONFIG_HIDEINT,
+	N_("The width of the size column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column"), NULL, 0},
+  {"remote_user_width", "", &remote_columns[2], CONFIG_HIDEINT,
+	N_("The width of the user column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column"), NULL, 0},
+  {"remote_group_width", "", &remote_columns[3], CONFIG_HIDEINT,
+	N_("The width of the group column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column"), NULL, 0},
+  {"remote_date_width", "", &remote_columns[4], CONFIG_HIDEINT,
+	N_("The width of the date column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column"), NULL, 0},
+  {"remote_attribs_width", "", &remote_columns[5], CONFIG_HIDEINT,
+	N_("The width of the attribs column in the file listboxes. Set this to 0 to have this column automagically resize. Set this to -1 to disable this column"), NULL, 0},
+  {"send_color", "", &send_color, CONFIG_COLOR,
+        N_("The color of the commands that are sent to the server"), NULL, 0},
+  {"recv_color", "", &recv_color, CONFIG_COLOR,
+        N_("The color of the commands that are received from the server"), NULL, 0},
+  {"error_color", "", &error_color, CONFIG_COLOR,
+        N_("The color of the error messages"), NULL, 0},
+  {"misc_color", "", &misc_color, CONFIG_COLOR,
+        N_("The color of the rest of the log messages"), NULL, 0},
+  {NULL, NULL, NULL, 0, "", NULL, 0}
 };
 
 
@@ -247,7 +248,7 @@ gftp_read_config_file (char **argv, int get_xpms)
       temp1str = expand_path (BASE_CONF_DIR);
       if (access (temp1str, F_OK) == -1)
 	{
-	  if (mkdir (temp1str, 0700) != 0)
+	  if (mkdir (temp1str, S_IRUSR | S_IWUSR | S_IXUSR) != 0)
 	    {
 	      printf (_("gFTP Error: Could not make directory %s: %s\n"),
 		      temp1str, g_strerror (errno));
@@ -267,7 +268,7 @@ gftp_read_config_file (char **argv, int get_xpms)
       copyfile (temp1str, tempstr);
       g_free (temp1str);
     }
-  chmod (tempstr, S_IRUSR | S_IWUSR);
+
   if ((conffile = fopen (tempstr, "r")) == NULL)
     {
       printf (_("gFTP Error: Cannot open config file %s: %s\n"), CONFIG_FILE,
@@ -287,7 +288,7 @@ gftp_read_config_file (char **argv, int get_xpms)
       len = strlen (buf);
       if (buf[len - 1] == '\n')
 	buf[--len] = '\0';
-      if (buf[len - 1] == '\r')
+      if (len && buf[len - 1] == '\r')
 	buf[--len] = '\0';
       line++;
 
@@ -473,8 +474,7 @@ gftp_read_config_file (char **argv, int get_xpms)
           ssh_extra_params_list = g_realloc (ssh_extra_params_list,
                                              (num_ssh_extra_params + 1) * 
                                              sizeof (char *));
-          ssh_extra_params_list[num_ssh_extra_params - 1] = 
-                                                  g_strconcat (curpos, NULL);
+          ssh_extra_params_list[num_ssh_extra_params - 1] = g_strdup (curpos);
           ssh_extra_params_list[num_ssh_extra_params] = NULL;
           *endpos = ' ';
           curpos = endpos + 1;
@@ -486,8 +486,7 @@ gftp_read_config_file (char **argv, int get_xpms)
           ssh_extra_params_list = g_realloc (ssh_extra_params_list,
                                              (num_ssh_extra_params + 1) * 
                                              sizeof (char *));
-          ssh_extra_params_list[num_ssh_extra_params - 1] = 
-                                                  g_strconcat (curpos, NULL);
+          ssh_extra_params_list[num_ssh_extra_params - 1] = g_strdup (curpos);
           ssh_extra_params_list[num_ssh_extra_params] = NULL;
         }
     }
@@ -520,11 +519,6 @@ gftp_read_config_file (char **argv, int get_xpms)
         emailaddr = g_strconcat (pw->pw_name, "@", unme.nodename, NULL);
       gftp_write_config_file ();
     }
-
-  if ((tempstr = getenv ("TMP")) != NULL && *tempstr != '\0')
-    tmp_directory = g_strconcat (tempstr, NULL);
-  else
-    tmp_directory = g_strconcat ("/tmp", NULL);
 }
 
 
@@ -545,18 +539,6 @@ gftp_read_bookmarks (void)
 
   if (access (tempstr, F_OK) == -1)
     {
-      temp1str = expand_path (BASE_CONF_DIR);
-      if (access (temp1str, F_OK) == -1)
-	{
-	  if (mkdir (temp1str, 0700) != 0)
-	    {
-	      printf (_("gFTP Error: Could not make directory %s: %s\n"),
-		      temp1str, g_strerror (errno));
-	      exit (-1);
-	    }
-	}
-      g_free (temp1str);
-
       temp1str = g_strdup_printf ("%s/bookmarks", SHARE_DIR);
       if (access (temp1str, F_OK) == -1)
 	{
@@ -568,7 +550,7 @@ gftp_read_bookmarks (void)
       copyfile (temp1str, tempstr);
       g_free (temp1str);
     }
-  chmod (tempstr, S_IRUSR | S_IWUSR);
+
   if ((bmfile = fopen (tempstr, "r")) == NULL)
     {
       printf (_("gFTP Error: Cannot open bookmarks file %s: %s\n"), tempstr,
@@ -584,7 +566,7 @@ gftp_read_bookmarks (void)
      len = strlen (buf);
       if (buf[len - 1] == '\n')
 	buf[--len] = '\0';
-      if (buf[len - 1] == '\r')
+      if (len && buf[len - 1] == '\r')
 	buf[--len] = '\0';
       line++;
 
@@ -763,18 +745,16 @@ add_to_bookmark (gftp_bookmarks * newentry)
 void
 gftp_write_config_file (void)
 {
-  char *hdr, *proxyhdr, *exthdr, *histhdr, *bmhdr;
+  char *hdr, *proxyhdr, *exthdr, *histhdr;
   gftp_file_extensions *tempext;
-  gftp_bookmarks *tempentry;
   gftp_proxy_hosts *hosts;
-  FILE *conffile, *bmfile;
-  char *tempstr, *str;
   gftp_color *color;
   GList *templist;
+  FILE *conffile;
+  char *tempstr;
   int pos;
 
   hdr = N_("Config file for gFTP. Copyright (C) 1998-2002 Brian Masney <masneyb@gftp.org>. Warning: Any comments that you add to this file WILL be overwritten. If a entry has a (*) in it's comment, you can't change it inside gFTP");
-  bmhdr = N_("Bookmarks file for gFTP. Copyright (C) 1998-2002 Brian Masney <masneyb@gftp.org>. Warning: Any comments that you add to this file WILL be overwritten");
   proxyhdr = N_("This section specifies which hosts are on the local subnet and won't need to go out the proxy server (if available). Syntax: dont_use_proxy=.domain or dont_use_proxy=network number/netmask");
   exthdr = N_("ext=file extenstion:XPM file:Ascii or Binary (A or B):viewer program. Note: All arguments except the file extension are optional");
   histhdr = N_("This section contains the data that is in the history");
@@ -784,33 +764,19 @@ gftp_write_config_file (void)
       printf (_("gFTP Error: Bad config file name %s\n"), CONFIG_FILE);
       exit (0);
     }
+
   if ((conffile = fopen (tempstr, "w+")) == NULL)
     {
       printf (_("gFTP Error: Cannot open config file %s: %s\n"), CONFIG_FILE,
 	      g_strerror (errno));
       exit (0);
     }
+
   g_free (tempstr);
 
   write_comment (conffile, _(hdr));
   fwrite ("\n", 1, 1, conffile);
 
-  if ((tempstr = expand_path (BOOKMARKS_FILE)) == NULL)
-    {
-      printf (_("gFTP Error: Bad bookmarks file name %s\n"), CONFIG_FILE);
-      exit (0);
-    }
-  if ((bmfile = fopen (tempstr, "w+")) == NULL)
-    {
-      printf (_("gFTP Error: Cannot open bookmarks file %s: %s\n"),
-	      CONFIG_FILE, g_strerror (errno));
-      exit (0);
-    }
-  g_free (tempstr);
-
-  write_comment (bmfile, _(bmhdr));
-  fwrite ("\n", 1, 1, bmfile);
-
   for (pos = 0; config_file_vars[pos].var != NULL; pos++)
     {
       if (config_file_vars[pos].type == CONFIG_CHARTEXT ||
@@ -854,46 +820,6 @@ gftp_write_config_file (void)
         }
     }
 
-  tempentry = bookmarks->children;
-  while (tempentry != NULL)
-    {
-      if (tempentry->children != NULL)
-	{
-	  tempentry = tempentry->children;
-	  continue;
-	}
-      str = tempentry->path;
-      while (*str == '/')
-	str++;
-      fprintf (bmfile,
-	       "[%s]\nhostname=%s\nport=%d\nprotocol=%s\nremote directory=%s\nlocal directory=%s\nusername=%s\npassword=%s\naccount=%s\n",
-	       str, tempentry->hostname == NULL ? "" : tempentry->hostname,
-	       tempentry->port, tempentry->protocol == NULL
-	       || *tempentry->protocol ==
-	       '\0' ? gftp_protocols[0].name : tempentry->protocol,
-	       tempentry->remote_dir == NULL ? "" : tempentry->remote_dir,
-	       tempentry->local_dir == NULL ? "" : tempentry->local_dir,
-	       tempentry->user == NULL ? "" : tempentry->user,
-	       !tempentry->save_password
-	       || tempentry->pass == NULL ? "" : tempentry->pass,
-	       tempentry->acct == NULL ? "" : tempentry->acct);
-
-      if (tempentry->sftpserv_path)
-        fprintf (bmfile, "sftpserv_path=%s\n", tempentry->sftpserv_path);
-
-      fprintf (bmfile, "\n");
- 
-      if (tempentry->next == NULL)
-	{
-	  tempentry = tempentry->prev;
-	  while (tempentry->next == NULL && tempentry->prev != NULL)
-	    tempentry = tempentry->prev;
-	  tempentry = tempentry->next;
-	}
-      else
-	tempentry = tempentry->next;
-    }
-
   write_comment (conffile, _(proxyhdr));
   templist = proxy_hosts;
   while (templist != NULL)
@@ -943,6 +869,76 @@ gftp_write_config_file (void)
     fprintf (conffile, "userhistory=%s\n", (char *) templist->data);
 
   fclose (conffile);
+}
+
+
+void
+gftp_write_bookmarks_file (void)
+{
+  gftp_bookmarks *tempentry;
+  char *bmhdr, *tempstr;
+  FILE * bmfile;
+
+  bmhdr = N_("Bookmarks file for gFTP. Copyright (C) 1998-2002 Brian Masney <masneyb@gftp.org>. Warning: Any comments that you add to this file WILL be overwritten");
+
+  if ((tempstr = expand_path (BOOKMARKS_FILE)) == NULL)
+    {
+      printf (_("gFTP Error: Bad bookmarks file name %s\n"), CONFIG_FILE);
+      exit (0);
+    }
+
+  if ((bmfile = fopen (tempstr, "w+")) == NULL)
+    {
+      printf (_("gFTP Error: Cannot open bookmarks file %s: %s\n"),
+	      CONFIG_FILE, g_strerror (errno));
+      exit (0);
+    }
+
+  g_free (tempstr);
+
+  write_comment (bmfile, _(bmhdr));
+  fwrite ("\n", 1, 1, bmfile);
+
+  tempentry = bookmarks->children;
+  while (tempentry != NULL)
+    {
+      if (tempentry->children != NULL)
+	{
+	  tempentry = tempentry->children;
+	  continue;
+	}
+      tempstr = tempentry->path;
+      while (*tempstr == '/')
+	tempstr++;
+      fprintf (bmfile,
+	       "[%s]\nhostname=%s\nport=%d\nprotocol=%s\nremote directory=%s\nlocal directory=%s\nusername=%s\npassword=%s\naccount=%s\n",
+	       tempstr, tempentry->hostname == NULL ? "" : tempentry->hostname,
+	       tempentry->port, tempentry->protocol == NULL
+	       || *tempentry->protocol ==
+	       '\0' ? gftp_protocols[0].name : tempentry->protocol,
+	       tempentry->remote_dir == NULL ? "" : tempentry->remote_dir,
+	       tempentry->local_dir == NULL ? "" : tempentry->local_dir,
+	       tempentry->user == NULL ? "" : tempentry->user,
+	       !tempentry->save_password
+	       || tempentry->pass == NULL ? "" : tempentry->pass,
+	       tempentry->acct == NULL ? "" : tempentry->acct);
+
+      if (tempentry->sftpserv_path)
+        fprintf (bmfile, "sftpserv_path=%s\n", tempentry->sftpserv_path);
+
+      fprintf (bmfile, "\n");
+ 
+      if (tempentry->next == NULL)
+	{
+	  tempentry = tempentry->prev;
+	  while (tempentry->next == NULL && tempentry->prev != NULL)
+	    tempentry = tempentry->prev;
+	  tempentry = tempentry->next;
+	}
+      else
+	tempentry = tempentry->next;
+    }
+
   fclose (bmfile);
 }
 
@@ -952,9 +948,6 @@ write_comment (FILE * fd, const char *comment)
 {
   const char *pos, *endpos;
 
-  /* FIXME - need to use UTF functions here. I need to check to see how 
-     portable they are before I do this though */
-
   fwrite ("# ", 1, 2, fd);
   pos = comment;
   while (strlen (pos) > 76)
