@@ -17,6 +17,8 @@
 /*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
 /*****************************************************************************/
 
+/* $Id: gftp.h,v 1.15 2002/11/27 14:29:55 masneyb Exp $ */
+
 #ifndef __GFTP_H
 #define __GFTP_H
 
@@ -33,6 +35,7 @@
 #endif
 #include <sys/wait.h>
 #include <sys/utsname.h>
+#include <sys/sysmacros.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <glib.h>
@@ -74,6 +77,18 @@
 #define AF_LOCAL AF_UNIX
 #endif
 
+/* We need the major() and minor() macros in the user interface. If they aren't
+   defined by the system, we'll just define them here. */
+#ifndef major
+#warning major macro was not defined by the system. Defining one that is probably wrong for your system
+#define major(dev) (((dev) >> 8) & 0xff)
+#endif
+
+#ifndef minor
+#warning minor macro was not defined by the system. Defining one that is probably wrong for your system
+#define minor(dev) ((dev) & 0xff)
+#endif
+
 #ifdef HAVE_DMALLOC
 #include <dmalloc.h>
 #endif
@@ -114,6 +129,13 @@ typedef struct gftp_file_tag gftp_file;
 #define GFTP_DIRECTION_DOWNLOAD			0
 #define GFTP_DIRECTION_UPLOAD			1
 
+#define GFTP_SORT_COL_FILE			1
+#define GFTP_SORT_COL_SIZE			2
+#define GFTP_SORT_COL_USER			3
+#define GFTP_SORT_COL_GROUP			4
+#define GFTP_SORT_COL_DATETIME			5
+#define GFTP_SORT_COL_ATTRIBS			6
+
 struct gftp_file_tag 
 {
   char *file,			/* Our filename */
@@ -123,8 +145,8 @@ struct gftp_file_tag
        *destfile;		/* Full pathname to the destination for the 
                                    file transfer */
   time_t datetime;		/* File date and time */
-  unsigned long size,		/* Size of the file */
-                startsize;	/* Size to start the transfer at */
+  off_t size,			/* Size of the file */
+        startsize;		/* Size to start the transfer at */
   unsigned int isdir : 1,	/* File type */
                isexe : 1,
                islink : 1,
@@ -143,7 +165,7 @@ struct gftp_file_tag
                is_fd : 1;	/* Is this a file descriptor? */
   char transfer_action;		/* See the GFTP_TRANS_ACTION_* vars above */
   void *node;			/* Pointer to the node for the gui */
-  FILE * fd;
+  int fd;
 };
 
 
@@ -180,10 +202,9 @@ struct gftp_request_tag
   unsigned int port,		/* Port of remote site */
                proxy_port;	/* Port of the proxy server */
 
-  FILE *sockfd,			/* Control connection (read) */
-       *sockfd_write,		/* Control connection (write) */
-       *datafd,			/* Data connection */
-       *cachefd;		/* For the directory cache */
+  int sockfd,			/* Control connection (read) */
+      datafd,			/* Data connection */
+      cachefd;			/* For the directory cache */
   int wakeup_main_thread[2];	/* FD that gets written to by the threads
                                    to wakeup the parent */
         
@@ -202,6 +223,8 @@ struct gftp_request_tag
                use_threads : 1,         /* Whether we need to spawn a thread
                                            for this protocol */
                cached : 1,              /* Is this directory listing cached? */
+               cancel : 1,		/* If a signal is received, should
+					   we cancel this operation */
                stopable : 1;
 
   off_t gotbytes;
@@ -215,13 +238,13 @@ struct gftp_request_tag
   void (*destroy)			( gftp_request * request );
   int (*connect)			( gftp_request * request );
   void (*disconnect) 			( gftp_request * request );
-  long (*get_file) 			( gftp_request * request, 
+  off_t (*get_file) 			( gftp_request * request, 
 					  const char *filename, 
-					  FILE * fd,
+					  int fd,
 					  off_t startsize );
   int (*put_file) 			( gftp_request * request, 
 					  const char *filename, 
-					  FILE * fd,
+					  int fd,
 					  off_t startsize,
 					  off_t totalsize );
   long (*transfer_file) 		( gftp_request * fromreq, 
@@ -230,17 +253,18 @@ struct gftp_request_tag
 					  gftp_request * toreq, 
 					  const char *tofile, 
 					  off_t tosize );
-  size_t (*get_next_file_chunk) 	( gftp_request * request, 
+  ssize_t (*get_next_file_chunk) 	( gftp_request * request, 
 					  char *buf, 
 					  size_t size );
-  size_t (*put_next_file_chunk) 	( gftp_request * request, 
+  ssize_t (*put_next_file_chunk) 	( gftp_request * request, 
 					  char *buf, 
 					  size_t size );
   int (*end_transfer) 			( gftp_request * request );
+  int (*abort_transfer) 		( gftp_request * request );
   int (*list_files) 			( gftp_request * request );
   int (*get_next_file)			( gftp_request * request, 
 					  gftp_file *fle, 
-					  FILE *fd );
+					  int fd );
   int (*set_data_type)			( gftp_request * request, 
 					  int data_type );
   off_t (*get_file_size) 		( gftp_request * request, 
@@ -266,6 +290,9 @@ struct gftp_request_tag
 					  const char *filename );
   int (*parse_url)			( gftp_request * request,
 					  const char *url );
+  void (*set_config_options)		( gftp_request * request );
+  void (*swap_socks)			( gftp_request * dest,
+					  gftp_request * source );
 
   /* Options */
   gftp_transfer_type transfer_type;	/* Passive or non-passive (FTP only) */
@@ -293,9 +320,7 @@ struct gftp_transfer_tag
                show : 1,
                stalled : 1,
                next_file : 1,
-               skip_file : 1,
-               current_file_number, 
-               current_file_retries;
+               skip_file : 1;
 
   struct timeval starttime,
                  lasttime;
@@ -307,7 +332,9 @@ struct gftp_transfer_tag
         * updfle;
 
   long numfiles,
-       numdirs;
+       numdirs,
+       current_file_number,
+       current_file_retries;
 
   off_t curtrans,		/* Current transfered bytes for this file */
         curresumed,		/* Resumed bytes for this file */
@@ -394,10 +421,10 @@ typedef struct gftp_config_vars_tag
   char *key,			/* variable name */
        *description;		/* How this field will show up in the dialog */
   gpointer var;			/* Pointer to our variable */
-  int type,			/* See defines below */
-      shown;			/* Is this shown in the options dialog? */
+  int type;			/* See defines below */
   char *comment;                /* Comment to write out to the config file */
   gpointer widget;
+  int ports_shown;		/* What ports of gFTP is this option shown in */
 } gftp_config_vars;
 
 #define CONFIG_INTTEXT                  1
@@ -414,6 +441,10 @@ typedef struct gftp_config_vars_tag
 #define CONFIG_UINTTEXT                 12
 #define CONFIG_CHARPASS			13
 
+#define GFTP_PORT_GTK			(1 << 1)
+#define GFTP_PORT_TEXT			(1 << 2)
+#define GFTP_PORT_ALL			(GFTP_PORT_GTK | GFTP_PORT_TEXT)
+
 typedef struct gftp_proxy_type_tag
 {
   char *key,
@@ -422,12 +453,20 @@ typedef struct gftp_proxy_type_tag
 
 #define GFTP_CUSTOM_PROXY_NUM        8
 
+typedef struct gftp_getline_buffer_tag
+{
+  char *buffer,
+       *curpos;
+  size_t max_bufsize,
+         cur_bufsize;
+} gftp_getline_buffer;
+
 /* Global config options */
 extern supported_gftp_protocols gftp_protocols[];
 extern char version[], *emailaddr, *edit_program, *view_program, 
             *firewall_host, *firewall_username, *firewall_password, 
             *firewall_account, *proxy_config, *http_proxy_host, 
-            *http_proxy_username, *http_proxy_password, *tmp_directory, 
+            *http_proxy_username, *http_proxy_password, 
             *startup_directory, *ssh_prog_name, *ssh_extra_params, 
             **ssh_extra_params_list, *default_protocol, *ssh1_sftp_path, 
             *ssh2_sftp_path;
@@ -446,7 +485,8 @@ extern int do_one_transfer_at_a_time, start_file_transfers,
            local_columns[6], remote_columns[6], resolve_symlinks, 
            firewall_port, http_proxy_port, overwrite_by_default, 
            append_file_transfers, enable_old_ssh, ssh_need_userpass, 
-           ssh_use_askpass, sshv2_use_sftp_subsys;
+           ssh_use_askpass, sshv2_use_sftp_subsys, local_sortcol, 
+           local_sortasds, remote_sortcol, remote_sortasds;
 extern guint max_log_window_size;
 extern GHashTable * bookmarks_htable, * config_htable;
 extern GList * localhistory, * remotehistory, * host_history, * port_history, 
@@ -458,13 +498,14 @@ extern gftp_proxy_type proxy_type[];
 extern gftp_color send_color, recv_color, error_color, misc_color;
 
 /* cache.c */
-FILE * gftp_new_cache_entry 		( gftp_request * request );
+int gftp_new_cache_entry 		( gftp_request * request );
 
-FILE * gftp_find_cache_entry 		( gftp_request * request );
+int gftp_find_cache_entry 		( gftp_request * request );
 
 void gftp_clear_cache_files 		( void );
 
-void gftp_delete_cache_entry 		( gftp_request * request );
+void gftp_delete_cache_entry 		( gftp_request * request,
+					  int ignore_directory );
 
 /* config_file.c */
 void gftp_read_config_file 		( char **argv,
@@ -476,12 +517,14 @@ void add_to_bookmark                    ( gftp_bookmarks *newentry );
 
 void gftp_write_config_file 		( void );
 
+void gftp_write_bookmarks_file 		( void );
+
 GHashTable * build_bookmarks_hash_table	( gftp_bookmarks * entry );
 
 void print_bookmarks 			( gftp_bookmarks * bookmarks );
 
 /* misc.c */
-char *insert_commas 			( unsigned long number, 
+char *insert_commas 			( off_t number, 
 					  char *dest_str, 
 					  size_t dest_len );
 
@@ -521,9 +564,6 @@ void free_fdata				( gftp_file * fle );
 
 gftp_file * copy_fdata 			( gftp_file * fle );
 
-void swap_socks 			( gftp_request * dest, 
-					  gftp_request * source );
-
 int compare_request 			( gftp_request * request1, 
 					  gftp_request * request2, 
 					  int compare_dirs );
@@ -551,8 +591,11 @@ char * ssh_start_login_sequence 	( gftp_request * request,
 gint64 hton64				( gint64 val );
 #endif
 
+GList * gftp_sort_filelist 		( GList * filelist, 
+					  int column, 
+					  int asds );
+
 /* protocols.c */
-#define GFTP_CONNECTED(request)			(request->sockfd != NULL)
 #define GFTP_GET_HOSTNAME(request)		(request->hostname)
 #define GFTP_GET_USERNAME(request)		(request->username)
 #define GFTP_GET_PASSWORD(request)		(request->password)
@@ -587,15 +630,16 @@ gint64 hton64				( gint64 val );
 #define GFTP_TYPE_BINARY			1
 #define GFTP_TYPE_ASCII				2   
 #define GFTP_IS_CONNECTED(request)		((request) != NULL && \
-                                                 ((request)->sockfd != NULL || \
-                                                  (request)->cached))
+                                                 ((request)->sockfd > 0 || \
+                                                  (request)->cached || \
+                                                  (request)->always_connected))
 
 
 void rfc959_init 			( gftp_request * request );
 
 int rfc959_get_next_file 		( gftp_request * request, 
 					  gftp_file *fle, 
-					  FILE *fd );
+					  int fd );
 
 void rfc2068_init 			( gftp_request * request );
 
@@ -609,7 +653,8 @@ void bookmark_init 			( gftp_request * request );
 
 gftp_request *gftp_request_new 		( void );
 
-void gftp_request_destroy 		( gftp_request * request );
+void gftp_request_destroy 		( gftp_request * request,
+					  int free_request );
 
 void gftp_file_destroy 			( gftp_file *file );
 
@@ -617,31 +662,31 @@ int gftp_connect 			( gftp_request * request );
 
 void gftp_disconnect 			( gftp_request * request );
 
-size_t gftp_get_file 			( gftp_request * request, 
+off_t gftp_get_file 			( gftp_request * request, 
 					  const char *filename, 
-					  FILE * fd,
+					  int fd,
 					  size_t startsize );
 
 int gftp_put_file 			( gftp_request * request, 
 					  const char *filename, 
-					  FILE * fd,
+					  int fd,
 					  size_t startsize,
 					  size_t totalsize );
 
 long gftp_transfer_file 		( gftp_request *fromreq, 
 					  const char *fromfile, 
-					  FILE * fromfd,
+					  int fromfd,
 					  size_t fromsize, 
 					  gftp_request *toreq, 
 					  const char *tofile, 
-					  FILE * tofd,
+					  int tofd,
 					  size_t tosize );
 
-size_t gftp_get_next_file_chunk 	( gftp_request * request, 
+ssize_t gftp_get_next_file_chunk 	( gftp_request * request, 
 					  char *buf, 
 					  size_t size );
 
-size_t gftp_put_next_file_chunk 	( gftp_request * request, 
+ssize_t gftp_put_next_file_chunk 	( gftp_request * request, 
 					  char *buf, 
 					  size_t size );
 
@@ -656,6 +701,8 @@ int gftp_get_next_file 			( gftp_request * request,
 
 int gftp_end_transfer 			( gftp_request * request );
 
+int gftp_abort_transfer 		( gftp_request * request );
+
 int gftp_read_response 			( gftp_request * request );
 
 int gftp_set_data_type 			( gftp_request * request, 
@@ -721,7 +768,7 @@ char gftp_site_cmd 			( gftp_request * request,
 void gftp_set_proxy_config 		( gftp_request * request, 
 					  const char *proxy_config );
 
-long gftp_get_file_size 		( gftp_request * request, 
+off_t gftp_get_file_size 		( gftp_request * request, 
 					  const char *filename );
 
 int gftp_need_proxy 			( gftp_request * request,
@@ -760,15 +807,43 @@ struct hostent *r_gethostbyname 	( const char *name,
 					  struct hostent *result_buf, 
 					  int *h_errnop );
 
+#endif
+
 struct servent *r_getservbyname 	( const char *name, 
 					  const char *proto,
 					  struct servent *result_buf, 
 					  int *h_errnop );
-#endif
-
 void gftp_set_config_options 		( gftp_request * request );
 
 void print_file_list 			( GList * list );
 
+ssize_t gftp_get_line 			( gftp_request * request, 
+					  gftp_getline_buffer ** rbuf,
+					  char * str, 
+					  size_t len, 
+					  int fd );
+
+ssize_t gftp_read 			( gftp_request * request, 
+					  void *ptr, 
+					  size_t size, 
+					  int fd );
+
+ssize_t gftp_write 			( gftp_request * request, 
+					  const char *ptr, 
+					  size_t size, 
+					  int fd );
+
+ssize_t gftp_writefmt 			( gftp_request * request, 
+					  int fd, 
+					  const char *fmt, 
+					  ... );
+
+int gftp_set_sockblocking 		( gftp_request * request, 
+					  int fd, 
+					  int non_blocking );
+
+void gftp_swap_socks 			( gftp_request * dest, 
+					  gftp_request * source );
+
 #endif
 
