@@ -19,23 +19,24 @@
 
 #include "gftp.h"
 #include "options.h"
+static const char cvsid[] = "$Id: misc.c,v 1.15 2002/11/27 14:29:55 masneyb Exp $";
 
 char *
-insert_commas (unsigned long number, char *dest_str, size_t dest_len)
+insert_commas (off_t number, char *dest_str, size_t dest_len)
 {
   char *frompos, *topos, src[50], *dest;
   int len, num, rem, i;
 
   if (dest_str != NULL)
     *dest_str = '\0';
-  len = log10 (number) + 1;
+  len = (number > 0 ? log10 (number) : 0) + 2;
 
   if (len <= 0) 
     {
       if (dest_str != NULL)
         strncpy (dest_str, "0", dest_len);
       else
-        dest_str = g_strconcat ("0", NULL);
+        dest_str = g_strdup ("0");
       return (dest_str);
     }
 
@@ -54,7 +55,11 @@ insert_commas (unsigned long number, char *dest_str, size_t dest_len)
   else
     dest = dest_str;
 
+#if defined (_LARGEFILE_SOURCE)
+  g_snprintf (src, sizeof (src), "%lld", number);
+#else
   g_snprintf (src, sizeof (src), "%ld", number);
+#endif
 
   num = strlen (src) / 3 - 1;
   rem = strlen (src) % 3;
@@ -181,7 +186,7 @@ expand_path (const char *src)
       else if (strcmp (pos, ".") != 0)
 	{
 	  if (newstr == NULL)
-	    newstr = g_strconcat (pos - 1, NULL);
+	    newstr = g_strdup (pos - 1);
 	  else
 	    {
 	      tempstr = g_strconcat (newstr, pos - 1, NULL);
@@ -254,24 +259,43 @@ make_nonnull (char **str)
 int
 copyfile (char *source, char *dest)
 {
-  FILE *srcfd, *destfd;
+  int srcfd, destfd;
   char buf[8192];
-  size_t n;
+  ssize_t n;
 
-  if ((srcfd = fopen (source, "rb")) == NULL)
-    return (0);
-
-  if ((destfd = fopen (dest, "wb")) == NULL)
+  if ((srcfd = open (source, O_RDONLY)) == -1)
     {
-      fclose (srcfd);
-      return (0);
+      printf (_("Error: Cannot open local file %s: %s\n"),
+              source, g_strerror (errno));
+      exit (1);
     }
 
-  while ((n = fread (buf, 1, sizeof (buf), srcfd)) > 0)
-    fwrite (buf, 1, n, destfd);
+  if ((destfd = open (dest, O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR)) == -1)
+    {
+      printf (_("Error: Cannot open local file %s: %s\n"),
+              dest, g_strerror (errno));
+      close (srcfd);
+      exit (1);
+    }
 
-  fclose (srcfd);
-  fclose (destfd);
+  while ((n = read (srcfd, buf, sizeof (buf))) > 0)
+    {
+      if (write (destfd, buf, n) == -1)
+        {
+          printf (_("Error: Could not write to socket: %s\n"), 
+                  g_strerror (errno));
+          exit (1);
+        }
+    }
+
+  if (n == -1)
+    {
+      printf (_("Error: Could not read from socket: %s\n"), g_strerror (errno));
+      exit (1);
+    }
+
+  close (srcfd);
+  close (destfd);
 
   return (1);
 }
@@ -390,7 +414,14 @@ string_hash_compare (gconstpointer path1, gconstpointer path2)
 guint
 string_hash_function (gconstpointer key)
 {
-  return (((char *) key)[0] + ((char *) key)[1] + ((char *) key)[2]);
+  guint ret;
+  int i;
+
+  ret = 0;
+  for (i=0; ((char *) key)[0] != '\0' && i < 3; i++)
+    ret += ((char *) key)[i];
+
+  return (ret);
 }
 
 
@@ -424,8 +455,8 @@ free_fdata (gftp_file * fle)
     g_free (fle->attribs);
   if (fle->destfile)
     g_free (fle->destfile);
-  if (fle->fd)
-    fclose (fle->fd);
+  if (fle->fd > 0)
+    close (fle->fd);
   g_free (fle);
 }
 
@@ -471,23 +502,6 @@ copy_fdata (gftp_file * fle)
 }
 
 
-void
-swap_socks (gftp_request * dest, gftp_request * source)
-{
-  dest->sockfd = source->sockfd;
-  dest->datafd = source->datafd;
-  dest->sockfd_write = source->sockfd_write;
-  dest->cached = 0;
-  if (!source->always_connected)
-    {
-      source->sockfd = NULL;
-      source->datafd = NULL;
-      source->sockfd_write = NULL;
-      source->cached = 1;
-    }
-}
-
-
 int
 compare_request (gftp_request * request1, gftp_request * request2,
                  int compare_dirs)
@@ -542,9 +556,9 @@ free_tdata (gftp_transfer * tdata)
   if (tdata->structmutex)
     g_free (tdata->structmutex);
   if (tdata->fromreq != NULL)
-    gftp_request_destroy (tdata->fromreq);
+    gftp_request_destroy (tdata->fromreq, 1);
   if (tdata->toreq != NULL)
-    gftp_request_destroy (tdata->toreq);
+    gftp_request_destroy (tdata->toreq, 1);
   free_file_list (tdata->files);
   g_free (tdata);
 }
@@ -555,39 +569,32 @@ copy_request (gftp_request * req)
 {
   gftp_request * newreq;
 
-  newreq = g_malloc0 (sizeof (*newreq));
-  memcpy (newreq, req, sizeof (*newreq));
+  newreq = gftp_request_new ();
 
   if (req->hostname)
-    newreq->hostname = g_strconcat (req->hostname, NULL);
+    newreq->hostname = g_strdup (req->hostname);
   if (req->username)
-    newreq->username = g_strconcat (req->username, NULL);
+    newreq->username = g_strdup (req->username);
   if (req->password)
-    newreq->password = g_strconcat (req->password, NULL);
+    newreq->password = g_strdup (req->password);
   if (req->account)
-    newreq->account = g_strconcat (req->account, NULL);
+    newreq->account = g_strdup (req->account);
   if (req->directory)
-    newreq->directory = g_strconcat (req->directory, NULL);
+    newreq->directory = g_strdup (req->directory);
+  newreq->port = req->port;
+  newreq->data_type = req->data_type;
+  newreq->use_proxy = req->use_proxy;
+  gftp_set_proxy_config (newreq, req->proxy_config);
+  newreq->transfer_type = req->transfer_type;
+  newreq->network_timeout = req->network_timeout;
+  newreq->retries = req->retries;
+  newreq->sleep_time = req->sleep_time;
+  newreq->passive_transfer = req->passive_transfer;
+  newreq->maxkbs = req->maxkbs;
+  newreq->logging_function = req->logging_function;
 
-  newreq->url_prefix = NULL;
-  newreq->protocol_name = NULL;
-  newreq->sftpserv_path = NULL;
-  newreq->proxy_config = NULL;
-  newreq->proxy_hostname = NULL;
-  newreq->proxy_username = NULL;
-  newreq->proxy_password = NULL;
-  newreq->proxy_account = NULL;
-  newreq->last_ftp_response = NULL;
-  newreq->last_dir_entry = NULL;
-  newreq->sockfd = NULL;
-  newreq->sockfd_write = NULL;
-  newreq->datafd = NULL;
-  newreq->cachefd = NULL;
-  newreq->hostp = NULL;
-  newreq->protocol_data = NULL;
-  
-  if (req->proxy_config != NULL)
-    newreq->proxy_config = g_strconcat (req->proxy_config, NULL);
+  if (req->sftpserv_path != NULL)
+    newreq->sftpserv_path = g_strdup (req->sftpserv_path);
 
   req->init (newreq);
 
@@ -758,14 +765,15 @@ make_ssh_exec_args (gftp_request * request, char *execname,
                     int use_sftp_subsys, char *portstring)
 {
   char **args, *oldstr, *tempstr;
+  struct servent serv_struct;
   int i, j;
 
-  args = g_malloc (sizeof (char *) * (num_ssh_extra_params + 10));
+  args = g_malloc (sizeof (char *) * (num_ssh_extra_params + 15));
 
   args[0] = ssh_prog_name != NULL && *ssh_prog_name != '\0' ? 
             ssh_prog_name : "ssh";
   i = 1;
-  tempstr = g_strconcat (args[0], NULL);
+  tempstr = g_strdup (args[0]);
 
   if (ssh_extra_params_list != NULL)
     {
@@ -802,6 +810,13 @@ make_ssh_exec_args (gftp_request * request, char *execname,
       args[i++] = "-p";
       args[i++] = portstring;
     }
+  else
+    {
+      if (!r_getservbyname ("ssh", "tcp", &serv_struct, NULL))
+        request->port = 22;
+      else
+        request->port = ntohs (serv_struct.s_port);
+    }
 
   if (use_sftp_subsys)
     {
@@ -830,106 +845,108 @@ make_ssh_exec_args (gftp_request * request, char *execname,
   return (args);
 }
 
+#define SSH_LOGIN_BUFSIZE	200
+#define SSH_ERROR_BADPASS	-1
+#define SSH_ERROR_QUESTION	-2
 
 char *
 ssh_start_login_sequence (gftp_request * request, int fd)
 {
-  size_t rem, len, diff, lastdiff;
-  int flags, wrotepw, ok;
-  struct timeval tv;
-  char *tempstr;
-  fd_set rdfds;
+  char *tempstr, *pwstr, *tmppos;
+  size_t rem, len, diff, lastdiff, key_pos;
+  int wrotepw, ok;
   ssize_t rd;
 
-  rem = len = 100;
-  tempstr = g_malloc0 (len);
-  diff = lastdiff = 0;
+  rem = len = SSH_LOGIN_BUFSIZE;
+  tempstr = g_malloc0 (len + 1);
+  key_pos = diff = lastdiff = 0;
   wrotepw = 0;
   ok = 1;
 
-  if ((flags = fcntl (fd, F_GETFL, 0)) < 0) 
-    {
-      g_free (tempstr);
-      return (NULL);
-    }
+  if (gftp_set_sockblocking (request, fd, 1) == -1)
+    return (NULL);
 
-  if (fcntl (fd, F_SETFL, flags | O_NONBLOCK) < 0)
-    {
-      g_free (tempstr);
-      return (NULL);
-    }
+  pwstr = g_strconcat (request->password, "\n", NULL);
 
+  errno = 0;
   while (1)
     {
-      FD_ZERO (&rdfds);
-      FD_SET (fd, &rdfds);
-      tv.tv_sec = 5;
-      tv.tv_usec = 0;
-      if (select (fd + 1, &rdfds, NULL, NULL, &tv) < 0)
+      if ((rd = gftp_read (request, tempstr + diff, rem - 1, fd)) <= 0)
         {
-          if (errno == EINTR)
-            continue;
           ok = 0;
           break;
         }
-
-      if ((rd = read (fd, tempstr + diff, rem - 1)) < 0)
-        {
-          if (errno == EINTR)
-            continue;
-          ok = 0;
-          break;
-        }
-      else if (rd == 0)
-        { 
-          ok = 0;
-          break;
-        }
-      tempstr[diff + rd] = '\0';
       rem -= rd;
       diff += rd;
-      if (rem <= 1)
+      tempstr[diff] = '\0'; 
+
+      if (diff > 11 && strcmp (tempstr + diff - 10, "password: ") == 0)
+        {
+          if (wrotepw)
+            {
+              ok = SSH_ERROR_BADPASS;
+              break;
+            }
+              
+          wrotepw = 1;
+          if (gftp_write (request, pwstr, strlen (pwstr), fd) < 0)
+            {
+              ok = 0;
+              break;
+            }
+        }
+      else if (diff > 2 && strcmp (tempstr + diff - 2, ": ") == 0 &&
+               ((tmppos = strstr (tempstr + key_pos, "Enter passphrase for RSA key")) != NULL ||
+                ((tmppos = strstr (tempstr + key_pos, "Enter passphrase for key '")) != NULL)))
+        {
+          key_pos = diff;
+          if (wrotepw)
+            {
+              ok = SSH_ERROR_BADPASS;
+              break;
+            }
+
+          wrotepw = 1;
+          if (gftp_write (request, pwstr, strlen (pwstr), fd) < 0)
+            {
+              ok = 0;
+              break;
+            }
+        }
+      else if (diff > 10 && strcmp (tempstr + diff - 10, "(yes/no)? ") == 0)
+        {
+          ok = SSH_ERROR_QUESTION;
+          break;
+        }
+      else if (diff >= 5 && strcmp (tempstr + diff - 5, "xsftp") == 0)
+        break;
+      else if (rem <= 1)
         {
-          tempstr = g_realloc (tempstr, len + 100);
-          tempstr[diff] = '\0';
           request->logging_function (gftp_logging_recv, request->user_data,
                                      "%s", tempstr + lastdiff);
+          len += SSH_LOGIN_BUFSIZE;
+          rem += SSH_LOGIN_BUFSIZE;
           lastdiff = diff;
-          len += 100;
-          rem = 100;
+          tempstr = g_realloc (tempstr, len);
+          continue;
         }
-
-      if (!wrotepw && 
-          strlen (tempstr) > 11 && strcmp (tempstr + strlen (tempstr) - 10, 
-                                           "password: ") == 0)
-        {
-          wrotepw = 1;
-          write (fd, request->password, strlen (request->password));
-          write (fd, "\n", 1);
-        }
-
-      else if (!wrotepw && 
-               (strstr (tempstr, "Enter passphrase for RSA key") != NULL ||
-               strstr (tempstr, "Enter passphrase for key '") != NULL))
-        {
-          wrotepw = 1;
-          write (fd, request->password, strlen (request->password));
-          write (fd, "\n", 1);
-        }
-      else if (strlen (tempstr) >= 5 && 
-               strcmp (tempstr + strlen (tempstr) - 5, "xsftp") == 0)
-        break;
     }
 
-  tempstr[diff] = '\0';
-  request->logging_function (gftp_logging_recv, request->user_data,
-                             "%s\n", tempstr + lastdiff);
+  g_free (pwstr);
 
-  if (ok && fcntl (fd, F_SETFL, flags) < 0)
-    ok = 0;
+  if (*(tempstr + lastdiff) != '\0')
+    request->logging_function (gftp_logging_recv, request->user_data,
+                               "%s\n", tempstr + lastdiff);
 
-  if (!ok)
+  if (ok <= 0)
     {
+      if (ok == SSH_ERROR_BADPASS)
+        request->logging_function (gftp_logging_error, request->user_data,
+                               _("Error: An incorrect password was entered\n"));
+      else if (ok == SSH_ERROR_QUESTION)
+        request->logging_function (gftp_logging_error, request->user_data,
+                               _("Please connect to this host with the command line SSH utility and answer this question appropriately.\n"));
+
       g_free (tempstr);
       return (NULL);
     }
@@ -961,3 +978,248 @@ hton64 (gint64 val)
 
 #endif
 
+
+static gint
+gftp_file_sort_function_as (gconstpointer a, gconstpointer b)
+{
+  const gftp_file * f1, * f2;
+
+  f1 = a;
+  f2 = b;
+  return (strcmp (f1->file, f2->file));
+}
+
+
+static gint
+gftp_file_sort_function_ds (gconstpointer a, gconstpointer b)
+{
+  const gftp_file * f1, * f2;
+  gint ret;
+
+  f1 = a;
+  f2 = b;
+  ret = strcmp (f1->file, f2->file);
+  if (ret < 0)
+    ret = 1;
+  else if (ret > 0)
+    ret = -1;
+  return (ret);
+}
+
+
+static gint
+gftp_user_sort_function_as (gconstpointer a, gconstpointer b)
+{
+  const gftp_file * f1, * f2;
+
+  f1 = a;
+  f2 = b;
+  return (strcmp (f1->user, f2->user));
+}
+
+
+static gint
+gftp_user_sort_function_ds (gconstpointer a, gconstpointer b)
+{
+  const gftp_file * f1, * f2;
+  gint ret;
+
+  f1 = a;
+  f2 = b;
+  ret = strcmp (f1->user, f2->user);
+  if (ret < 0)
+    ret = 1;
+  else if (ret > 0)
+    ret = -1;
+  return (ret);
+}
+
+
+static gint
+gftp_group_sort_function_as (gconstpointer a, gconstpointer b)
+{
+  const gftp_file * f1, * f2;
+
+  f1 = a;
+  f2 = b;
+  return (strcmp (f1->group, f2->group));
+}
+
+
+static gint
+gftp_group_sort_function_ds (gconstpointer a, gconstpointer b)
+{
+  const gftp_file * f1, * f2;
+  gint ret;
+
+  f1 = a;
+  f2 = b;
+  ret = strcmp (f1->group, f2->group);
+  if (ret < 0)
+    ret = 1;
+  else if (ret > 0)
+    ret = -1;
+  return (ret);
+}
+
+
+static gint
+gftp_attribs_sort_function_as (gconstpointer a, gconstpointer b)
+{
+  const gftp_file * f1, * f2;
+
+  f1 = a;
+  f2 = b;
+  return (strcmp (f1->attribs, f2->attribs));
+}
+
+
+static gint
+gftp_attribs_sort_function_ds (gconstpointer a, gconstpointer b)
+{
+  const gftp_file * f1, * f2;
+  gint ret;
+
+  f1 = a;
+  f2 = b;
+  ret = strcmp (f1->attribs, f2->attribs);
+  if (ret < 0)
+    ret = 1;
+  else if (ret > 0)
+    ret = -1;
+  return (ret);
+}
+
+
+static gint
+gftp_size_sort_function_as (gconstpointer a, gconstpointer b)
+{
+  const gftp_file * f1, * f2;
+
+  f1 = a;
+  f2 = b;
+  if (f1->size < f2->size)
+    return (-1);
+  else if (f1->size == f2->size)
+    return (0);
+  else
+    return (1);
+}
+
+
+static gint
+gftp_size_sort_function_ds (gconstpointer a, gconstpointer b)
+{
+  const gftp_file * f1, * f2;
+
+  f1 = a;
+  f2 = b;
+  if (f1->size < f2->size)
+    return (1);
+  else if (f1->size == f2->size)
+    return (0);
+  else
+    return (-1);
+}
+
+
+static gint
+gftp_datetime_sort_function_as (gconstpointer a, gconstpointer b)
+{
+  const gftp_file * f1, * f2;
+
+  f1 = a;
+  f2 = b;
+  if (f1->datetime < f2->datetime)
+    return (-1);
+  else if (f1->datetime == f2->datetime)
+    return (0);
+  else
+    return (1);
+}
+
+
+static gint
+gftp_datetime_sort_function_ds (gconstpointer a, gconstpointer b)
+{
+  const gftp_file * f1, * f2;
+
+  f1 = a;
+  f2 = b;
+  if (f1->datetime < f2->datetime)
+    return (1);
+  else if (f1->datetime == f2->datetime)
+    return (0);
+  else
+    return (-1);
+}
+
+
+GList *
+gftp_sort_filelist (GList * filelist, int column, int asds)
+{
+  GList * files, * dirs, * dotdot, * tempitem, * insitem;
+  GCompareFunc sortfunc;
+  gftp_file * tempfle;
+
+  files = dirs = dotdot = NULL;
+
+  if (column == GFTP_SORT_COL_FILE)
+    sortfunc = asds ?  gftp_file_sort_function_as : gftp_file_sort_function_ds;
+  else if (column == GFTP_SORT_COL_SIZE)
+    sortfunc = asds ?  gftp_size_sort_function_as : gftp_size_sort_function_ds;
+  else if (column == GFTP_SORT_COL_USER)
+    sortfunc = asds ?  gftp_user_sort_function_as : gftp_user_sort_function_ds;
+  else if (column == GFTP_SORT_COL_GROUP)
+    sortfunc = asds ?
+                gftp_group_sort_function_as : gftp_group_sort_function_ds;
+  else if (column == GFTP_SORT_COL_DATETIME)
+    sortfunc = asds ?
+                gftp_datetime_sort_function_as : gftp_datetime_sort_function_ds;  else /* GFTP_SORT_COL_ATTRIBS */
+    sortfunc = asds ? 
+                gftp_attribs_sort_function_as : gftp_attribs_sort_function_ds;
+
+  for (tempitem = filelist; tempitem != NULL; )
+    {
+      tempfle = tempitem->data;
+      insitem = tempitem;
+      tempitem = tempitem->next;
+      insitem->next = NULL;
+
+      if (dotdot == NULL && strcmp (tempfle->file, "..") == 0)
+        dotdot = insitem;
+      else if (sort_dirs_first && tempfle->isdir)
+        {
+          insitem->next = dirs;
+          dirs = insitem;
+        }
+      else
+        {
+          insitem->next = files;
+          files = insitem;
+        }
+    }
+
+  if (dirs != NULL)
+    dirs = g_list_sort (dirs, sortfunc);
+  if (files != NULL)
+    files = g_list_sort (files, sortfunc);
+
+  filelist = dotdot;
+
+  if (filelist == NULL)
+    filelist = dirs;
+  else
+    filelist = g_list_concat (filelist, dirs);
+
+  if (filelist == NULL)
+    filelist = files;
+  else
+    filelist = g_list_concat (filelist, files);
+
+  /* I haven't check this, but I'm pretty sure some older versions of glib
+     had a bug that the prev pointer wasn't being sent to NULL */
+  filelist->prev = NULL;
+  return (filelist);
+}
+
