@@ -18,27 +18,7 @@
 /*****************************************************************************/
 
 #include "gftp.h"
-
-static time_t parse_time 			( char **str );
-static int gftp_parse_ls_eplf 			( char *str, 
-						  gftp_file * fle );
-static int gftp_parse_ls_unix 			( char *str, 
-						  int cols, 
-						  gftp_file * fle );
-static int gftp_parse_ls_nt 			( char *str, 
-						  gftp_file * fle );
-static int gftp_parse_ls_novell 		( char *str, 
-						  gftp_file * fle );
-static char *copy_token 			( char **dest, 
-						  char *source );
-static char *goto_next_token 			( char *pos );
-static GList * gftp_get_dir_listing 		( gftp_transfer * transfer,
-						  int getothdir );
-static GHashTable * gftp_gen_dir_hash 		( gftp_request * request );
-static void gftp_destroy_dir_hash 		( GHashTable * dirhash );
-static void destroy_hash_ent 			( gpointer key, 
-						  gpointer value, 
-						  gpointer user_data );
+static const char cvsid[] = "$Id: protocols.c,v 1.19 2002/11/27 14:29:55 masneyb Exp $";
 
 gftp_request *
 gftp_request_new (void)
@@ -46,15 +26,16 @@ gftp_request_new (void)
   gftp_request *request;
 
   request = g_malloc0 (sizeof (*request));
-  request->sockfd = NULL;
-  request->datafd = NULL;
+  request->sockfd = -1;
+  request->datafd = -1;
+  request->cachefd = -1;
   request->data_type = GFTP_TYPE_BINARY;
   return (request);
 }
 
 
 void
-gftp_request_destroy (gftp_request * request)
+gftp_request_destroy (gftp_request * request, int free_request)
 {
   g_return_if_fail (request != NULL);
 
@@ -95,8 +76,18 @@ gftp_request_destroy (gftp_request * request)
     g_free (request->protocol_data);
   if (request->sftpserv_path)
     g_free (request->sftpserv_path);
+
   memset (request, 0, sizeof (*request));
-  g_free (request);
+
+  if (free_request)
+    g_free (request);
+  else
+    {
+      request->sockfd = -1;
+      request->datafd = -1;
+      request->cachefd = -1;
+      request->data_type = GFTP_TYPE_BINARY;
+    }
 }
 
 
@@ -127,19 +118,6 @@ gftp_connect (gftp_request * request)
   if (request->connect == NULL)
     return (-2);
 
-  if (request->sftpserv_path == NULL)
-    {
-      switch (request->protonum)
-        {
-          case GFTP_SSH_NUM:
-            request->sftpserv_path = g_strconcat (ssh1_sftp_path, NULL);
-            break;
-          case GFTP_SSHV2_NUM:
-            request->sftpserv_path = g_strconcat (ssh2_sftp_path, NULL);
-            break;
-        }
-    }
-
   gftp_set_config_options (request);
 
   return (request->connect (request));
@@ -170,8 +148,8 @@ gftp_disconnect (gftp_request * request)
 }
 
 
-size_t
-gftp_get_file (gftp_request * request, const char *filename, FILE * fd,
+off_t
+gftp_get_file (gftp_request * request, const char *filename, int fd,
                size_t startsize)
 {
   g_return_val_if_fail (request != NULL, -2);
@@ -184,7 +162,7 @@ gftp_get_file (gftp_request * request, const char *filename, FILE * fd,
 
 
 int
-gftp_put_file (gftp_request * request, const char *filename, FILE * fd,
+gftp_put_file (gftp_request * request, const char *filename, int fd,
                size_t startsize, size_t totalsize)
 {
   g_return_val_if_fail (request != NULL, -2);
@@ -198,9 +176,9 @@ gftp_put_file (gftp_request * request, const char *filename, FILE * fd,
 
 long
 gftp_transfer_file (gftp_request * fromreq, const char *fromfile, 
-                    FILE * fromfd, size_t fromsize, 
+                    int fromfd, size_t fromsize, 
                     gftp_request * toreq, const char *tofile,
-                    FILE * tofd, size_t tosize)
+                    int tofd, size_t tosize)
 {
   long size;
 
@@ -224,7 +202,9 @@ gftp_transfer_file (gftp_request * fromreq, const char *fromfile,
 
   if (gftp_put_file (toreq, tofile, tofd, tosize, size) != 0)
     {
-      gftp_end_transfer (fromreq);
+      if (gftp_abort_transfer (fromreq) != 0)
+        gftp_end_transfer (fromreq);
+
       return (-2);
     }
 
@@ -232,92 +212,32 @@ gftp_transfer_file (gftp_request * fromreq, const char *fromfile,
 }
 
 
-size_t 
+ssize_t 
 gftp_get_next_file_chunk (gftp_request * request, char *buf, size_t size)
 {
-  struct timeval tv;
-  fd_set fset;
-  size_t len;
-  
   g_return_val_if_fail (request != NULL, -2);
   g_return_val_if_fail (buf != NULL, -2);
 
   if (request->get_next_file_chunk != NULL)
     return (request->get_next_file_chunk (request, buf, size));
 
-  FD_ZERO (&fset);
-  FD_SET (fileno (request->datafd), &fset);
-  tv.tv_sec = request->network_timeout;
-  tv.tv_usec = 0;
-  if (select (fileno (request->datafd) + 1, &fset, NULL, NULL, &tv) <= 0)
-    {
-      request->logging_function (gftp_logging_error, request->user_data,
-                                 _("Connection to %s timed out\n"),
-                                 request->hostname);
-      gftp_disconnect (request);
-      return (-1);
-    }
-
-  if ((len = read (fileno (request->datafd), buf, size)) == -1)
-    {
-      request->logging_function (gftp_logging_error, request->user_data,
-                                 _("Error reading from host %s: %s\n"),
-                                 request->hostname,
-                                 g_strerror (errno));
-      gftp_disconnect (request);
-      return (-1);
-    }
-  return (len);
+  return (gftp_read (request, buf, size, request->datafd));
 }
 
 
-size_t 
+ssize_t 
 gftp_put_next_file_chunk (gftp_request * request, char *buf, size_t size)
 {
-  struct timeval tv;
-  size_t len, ret;
-  fd_set fset;
-  char *pos;
-  
   g_return_val_if_fail (request != NULL, -2);
+  g_return_val_if_fail (buf != NULL, -2);
 
-  ret = size;
   if (request->put_next_file_chunk != NULL)
     return (request->put_next_file_chunk (request, buf, size));
 
   if (size == 0)
     return (0);
 
-  pos = buf;
-  while (size > 0)
-    {
-      FD_ZERO (&fset);
-      FD_SET (fileno (request->datafd), &fset);
-      tv.tv_sec = request->network_timeout;
-      tv.tv_usec = 0;
-      if (!select (fileno (request->datafd) + 1, NULL, &fset, NULL, &tv))
-        {
-          request->logging_function (gftp_logging_error, request->user_data,
-			             _("Connection to %s timed out\n"),
-                                     request->hostname);
-          gftp_disconnect (request);
-          return (-1);
-        }
-
-      if ((len = write (fileno (request->datafd), pos, size)) < 0)
-        {
-          request->logging_function (gftp_logging_error, request->user_data,
-                                     _("Error writing to host %s: %s\n"),
-                                     request->hostname,
-                                     g_strerror (errno));
-          gftp_disconnect (request);
-          return (-1);
-        }
-      size -= len;
-      pos += len;
-    }
-
-  return (ret); 
+  return (gftp_write (request, buf, size, request->datafd));
 }
 
 
@@ -328,18 +248,17 @@ gftp_end_transfer (gftp_request * request)
 
   g_return_val_if_fail (request != NULL, -2);
 
-  if (request->end_transfer == NULL)
-    return (-2);
-
-  ret = 0;
-  if (request->cachefd != NULL)
-    {
-      fclose (request->cachefd);
-      request->cachefd = NULL;
-    }
-
-  if (!request->cached)
+  if (!request->cached && 
+      request->end_transfer != NULL)
     ret = request->end_transfer (request);
+  else
+    ret = 0;
+
+  if (request->cachefd > 0)
+    {
+      close (request->cachefd);
+      request->cachefd = -1;
+    }
 
   if (request->last_dir_entry)
     {
@@ -352,15 +271,27 @@ gftp_end_transfer (gftp_request * request)
 }
 
 
+int
+gftp_abort_transfer (gftp_request * request)
+{
+  g_return_val_if_fail (request != NULL, -2);
+
+  if (request->abort_transfer == NULL)
+    return (-2);
+
+  return (request->abort_transfer (request));
+}
+
+
 int
 gftp_list_files (gftp_request * request)
 {
-  FILE * fd;
+  int fd;
 
   g_return_val_if_fail (request != NULL, -2);
 
   request->cached = 0;
-  if (request->use_cache && (fd = gftp_find_cache_entry (request)) != NULL)
+  if (request->use_cache && (fd = gftp_find_cache_entry (request)) > 0)
     {
       request->logging_function (gftp_logging_misc, request->user_data,
                                  _("Loading directory listing %s from cache\n"),
@@ -385,15 +316,19 @@ gftp_list_files (gftp_request * request)
 int
 gftp_get_next_file (gftp_request * request, char *filespec, gftp_file * fle)
 {
-  FILE * fd;
-  int ret;
+  int fd, ret;
+#if GLIB_MAJOR_VERSION > 1
+  gsize bread, bwrite;
+  char *tempstr;
+  GError * error;
+#endif
 
   g_return_val_if_fail (request != NULL, -2);
 
   if (request->get_next_file == NULL)
     return (-2);
 
-  if (request->cached && request->cachefd != NULL)
+  if (request->cached && request->cachefd > 0)
     fd = request->cachefd;
   else
     fd = request->datafd;
@@ -404,18 +339,33 @@ gftp_get_next_file (gftp_request * request, char *filespec, gftp_file * fle)
       gftp_file_destroy (fle);
       ret = request->get_next_file (request, fle, fd);
 
-      if (ret >= 0 && !request->cached && request->cachefd != NULL && 
+#if GLIB_MAJOR_VERSION > 1
+      if (fle->file != NULL && !g_utf8_validate (fle->file, -1, NULL))
+        {
+          error = NULL;
+          if ((tempstr = g_locale_to_utf8 (fle->file, -1, &bread, 
+                                           &bwrite, &error)) != NULL)
+            {
+              g_free (fle->file);
+              fle->file = tempstr;
+            }
+          else
+            g_warning ("Error when converting %s to UTF-8: %s\n", fle->file,
+                       error->message);
+        }
+#endif
+
+      if (ret >= 0 && !request->cached && request->cachefd > 0 && 
           request->last_dir_entry != NULL)
         {
-          fwrite (request->last_dir_entry, 1, request->last_dir_entry_len, 
-                  request->cachefd);
-          if (ferror (request->cachefd))
+          if (gftp_write (request, request->last_dir_entry,
+                          request->last_dir_entry_len, request->cachefd) < 0)
             {
               request->logging_function (gftp_logging_error, request->user_data,
                                         _("Error: Cannot write to cache: %s\n"),
                                         g_strerror (errno));
-              fclose (request->cachefd);
-              request->cachefd = NULL;
+              close (request->cachefd);
+              request->cachefd = -1;
             }
         }
     } while (ret > 0 && !gftp_match_filespec (fle->file, filespec));
@@ -428,12 +378,17 @@ int
 gftp_parse_url (gftp_request * request, const char *url)
 {
   char *pos, *endpos, *endhostpos, *str, tempchar;
+  gftp_logging_func logging_function;
   const char *stpos;
   int len, i;
 
   g_return_val_if_fail (request != NULL, -2);
   g_return_val_if_fail (url != NULL, -2);
 
+  logging_function = request->logging_function;
+  gftp_request_destroy (request, 0);
+  request->logging_function = logging_function;
+
   for (stpos = url; *stpos == ' '; stpos++);
 
   if ((pos = strstr (stpos, "://")) != NULL)
@@ -470,7 +425,7 @@ gftp_parse_url (gftp_request * request, const char *url)
   if (i == GFTP_LOCAL_NUM)
     {
       request->directory = g_malloc (strlen (stpos + 6) + 1);
-      strcpy (request->directory, stpos + 6);
+      strcpy (request->directory, stpos + 7);
       return (0);
     }
 
@@ -510,16 +465,6 @@ gftp_parse_url (gftp_request * request, const char *url)
 	  pos = endpos + 1;
 	}
     }
-  else
-    {
-      gftp_set_username (request, "anonymous");
-      if (request->password)
-        {
-	  g_free (request->password);
-          request->password = NULL;
-        }
-      gftp_set_password (request, emailaddr);
-    }
 
   /* Now get the hostname and an optional port and optional directory */
   endhostpos = pos + strlen (pos);
@@ -615,7 +560,7 @@ gftp_set_directory (gftp_request * request, const char *directory)
   g_return_val_if_fail (directory != NULL, -2);
 
 
-  if (request->sockfd == NULL)
+  if (request->sockfd <= 0 && !request->always_connected)
     {
       if (directory != request->directory)
 	{
@@ -786,11 +731,16 @@ gftp_set_proxy_config (gftp_request * request, const char *proxy_config)
   int len;
 
   g_return_if_fail (request != NULL);
-  g_return_if_fail (proxy_config != NULL);
 
   if (request->proxy_config != NULL)
     g_free (request->proxy_config);
 
+  if (proxy_config == NULL)
+    {
+      request->proxy_config = NULL;
+      return;
+    }
+
   len = strlen (proxy_config);
 
   if (len > 0 && (proxy_config[len - 1] != 'n' ||
@@ -808,7 +758,7 @@ gftp_set_proxy_config (gftp_request * request, const char *proxy_config)
 }
 
 
-long
+off_t
 gftp_get_file_size (gftp_request * request, const char *filename)
 {
   g_return_val_if_fail (request != NULL, 0);
@@ -960,87 +910,45 @@ gftp_convert_ascii (char *buf, ssize_t *len, int direction)
 }
 
 
-int
-gftp_parse_ls (const char *lsoutput, gftp_file * fle)
+static char *
+copy_token (char **dest, char *source)
 {
-  int result, cols;
-  char *str, *pos;
+  /* This function is used internally by gftp_parse_ls () */
+  char *endpos, savepos;
 
-  g_return_val_if_fail (lsoutput != NULL, -2);
-  g_return_val_if_fail (fle != NULL, -2);
+  endpos = source;
+  while (*endpos != ' ' && *endpos != '\t' && *endpos != '\0')
+    endpos++;
+  if (*endpos == '\0')
+    return (NULL);
 
-  str = g_malloc (strlen (lsoutput) + 1);
-  strcpy (str, lsoutput);
-  memset (fle, 0, sizeof (*fle));
+  savepos = *endpos;
+  *endpos = '\0';
+  *dest = g_malloc (endpos - source + 1);
+  strcpy (*dest, source);
+  *endpos = savepos;
 
-  if (str[strlen (str) - 1] == '\n')
-    str[strlen (str) - 1] = '\0';
-  if (str[strlen (str) - 1] == '\r')
-    str[strlen (str) - 1] = '\0';
-  if (*lsoutput == '+') 			/* EPLF format */
-    result = gftp_parse_ls_eplf (str, fle);
-  else if (isdigit ((int) str[0]) && str[2] == '-') 	/* DOS/WinNT format */
-    result = gftp_parse_ls_nt (str, fle);
-  else if (str[1] == ' ' && str[2] == '[') 	/* Novell format */
-    result = gftp_parse_ls_novell (str, fle);
-  else
-    {
-      /* UNIX/MacOS format */
+  /* Skip the blanks till we get to the next entry */
+  source = endpos + 1;
+  while ((*source == ' ' || *source == '\t') && *source != '\0')
+    source++;
+  return (source);
+}
 
-      /* If there is no space between the attribs and links field, just make one */
-      if (strlen (str) > 10)
-	str[10] = ' ';
 
-      /* Determine the number of columns */
-      cols = 0;
-      pos = str;
-      while (*pos != '\0')
-	{
-	  while (*pos != '\0' && *pos != ' ' && *pos != '\t')
-	    {
-	      if (*pos == ':')
-		break;
-	      pos++;
-	    }
+static char *
+goto_next_token (char *pos)
+{
+  while (*pos != ' ' && *pos != '\t' && *pos != '\0')
+    pos++;
 
-	  cols++;
+  if (pos == '\0')
+    return (pos);
 
-	  if (*pos == ':')
-	    {
-	      cols++;
-	      break;
-	    }
+  while ((*pos == ' ' || *pos == '\t') && *pos != '\0')
+    pos++;
 
-	  while (*pos == ' ' || *pos == '\t')
-	    pos++;
-	}
-
-      if (cols > 6)
-	result = gftp_parse_ls_unix (str, cols, fle);
-      else
-	result = -2;
-    }
-  g_free (str);
-
-  if (fle->attribs == NULL)
-    return (result);
-
-  if (*fle->attribs == 'd')
-    fle->isdir = 1;
-  if (*fle->attribs == 'l')
-    fle->islink = 1;
-  if (strchr (fle->attribs, 'x') != NULL && !fle->isdir && !fle->islink)
-    fle->isexe = 1;
-  if (*fle->attribs == 'b')
-    fle->isblock = 1;
-  if (*fle->attribs == 'c')
-    fle->ischar = 1;
-  if (*fle->attribs == 's')
-    fle->issocket = 1;
-  if (*fle->attribs == 'p')
-    fle->isfifo = 1;
-
-  return (result);
+  return (pos);
 }
 
 
@@ -1048,8 +956,7 @@ static time_t
 parse_time (char **str)
 {
   const char *months[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",
-    "Aug", "Sep", "Oct", "Nov", "Dec"
-  };
+    "Aug", "Sep", "Oct", "Nov", "Dec" };
   char *startpos, *endpos, *datepos;
   struct tm curtime, tt;
   time_t t;
@@ -1244,6 +1151,7 @@ gftp_parse_ls_unix (char *str, int cols, gftp_file * fle)
       startpos = goto_next_token (startpos);
     }
 
+  /* FIXME - this is a bug if there is some extra spaces in the file */
   /* See if this is a Cray directory listing. It has the following format:
      drwx------     2 feiliu    g913     DK  common      4096 Sep 24  2001 wv */
   if (cols == 11 && strstr (str, "->") == NULL)
@@ -1255,7 +1163,7 @@ gftp_parse_ls_unix (char *str, int cols, gftp_file * fle)
   /* See if this is a block or character device. We will store the major number
      in the high word and the minor number in the low word.  */
   if ((fle->attribs[0] == 'b' || fle->attribs[0] == 'u' || 
-       fle->attribs[0] == 'c') && 
+       fle->attribs[0] == 'c') &&
       ((endpos = strchr (startpos, ',')) != NULL))
     {
       fle->size = strtol (startpos, NULL, 10) << 16;
@@ -1267,7 +1175,7 @@ gftp_parse_ls_unix (char *str, int cols, gftp_file * fle)
       /* Get the minor number */
       if ((endpos = strchr (startpos, ' ')) == NULL)
 	return (-2);
-      fle->size |= strtol (startpos, NULL, 10);
+      fle->size |= strtol (startpos, NULL, 10) & 0xFF;
     }
   else
     {
@@ -1373,39 +1281,138 @@ gftp_parse_ls_novell (char *str, gftp_file * fle)
 }
 
 
-static char *
-copy_token (char **dest, char *source)
+int
+gftp_parse_ls (const char *lsoutput, gftp_file * fle)
 {
-  /* This function is used internally by gftp_parse_ls () */
-  char *endpos;
+  int result, cols;
+  char *str, *pos;
 
-  endpos = source;
-  while (*endpos != ' ' && *endpos != '\t' && *endpos != '\0')
-    endpos++;
-  if (*endpos == '\0')
-    return (NULL);
-  *endpos = '\0';
-  *dest = g_malloc (endpos - source + 1);
-  strcpy (*dest, source);
+  g_return_val_if_fail (lsoutput != NULL, -2);
+  g_return_val_if_fail (fle != NULL, -2);
 
-  /* Skip the blanks till we get to the next entry */
-  source = endpos + 1;
-  while ((*source == ' ' || *source == '\t') && *source != '\0')
-    source++;
-  return (source);
+  str = g_malloc (strlen (lsoutput) + 1);
+  strcpy (str, lsoutput);
+  memset (fle, 0, sizeof (*fle));
+
+  if (str[strlen (str) - 1] == '\n')
+    str[strlen (str) - 1] = '\0';
+  if (str[strlen (str) - 1] == '\r')
+    str[strlen (str) - 1] = '\0';
+  if (*lsoutput == '+') 			/* EPLF format */
+    result = gftp_parse_ls_eplf (str, fle);
+  else if (isdigit ((int) str[0]) && str[2] == '-') 	/* DOS/WinNT format */
+    result = gftp_parse_ls_nt (str, fle);
+  else if (str[1] == ' ' && str[2] == '[') 	/* Novell format */
+    result = gftp_parse_ls_novell (str, fle);
+  else
+    {
+      /* UNIX/MacOS format */
+
+      /* If there is no space between the attribs and links field, just make one */
+      if (strlen (str) > 10)
+	str[10] = ' ';
+
+      /* Determine the number of columns */
+      cols = 0;
+      pos = str;
+      while (*pos != '\0')
+	{
+	  while (*pos != '\0' && *pos != ' ' && *pos != '\t')
+	    {
+	      if (*pos == ':')
+		break;
+	      pos++;
+	    }
+
+	  cols++;
+
+	  if (*pos == ':')
+	    {
+	      cols++;
+	      break;
+	    }
+
+	  while (*pos == ' ' || *pos == '\t')
+	    pos++;
+	}
+
+      if (cols > 6)
+	result = gftp_parse_ls_unix (str, cols, fle);
+      else
+	result = -2;
+    }
+  g_free (str);
+
+  if (fle->attribs == NULL)
+    return (result);
+
+  if (*fle->attribs == 'd')
+    fle->isdir = 1;
+  if (*fle->attribs == 'l')
+    fle->islink = 1;
+  if (strchr (fle->attribs, 'x') != NULL && !fle->isdir && !fle->islink)
+    fle->isexe = 1;
+  if (*fle->attribs == 'b')
+    fle->isblock = 1;
+  if (*fle->attribs == 'c')
+    fle->ischar = 1;
+  if (*fle->attribs == 's')
+    fle->issocket = 1;
+  if (*fle->attribs == 'p')
+    fle->isfifo = 1;
+
+  return (result);
 }
 
 
-static char *
-goto_next_token (char *pos)
+static GHashTable *
+gftp_gen_dir_hash (gftp_request * request)
 {
-  while (*pos != ' ' && *pos != '\t' && *pos != '\0')
-    pos++;
-  if (pos == '\0')
-    return (pos);
-  while ((*pos == ' ' || *pos == '\t') && *pos != '\0')
-    pos++;
-  return (pos);
+  unsigned long *newsize;
+  GHashTable * dirhash;
+  gftp_file * fle;
+  char * newname;
+
+
+  dirhash = g_hash_table_new (string_hash_function, string_hash_compare);
+  if (gftp_list_files (request) == 0)
+    {
+      fle = g_malloc0 (sizeof (*fle));
+      while (gftp_get_next_file (request, NULL, fle) > 0)
+        {
+          newname = fle->file;
+          newsize = g_malloc (sizeof (unsigned long));
+          *newsize = fle->size;
+          g_hash_table_insert (dirhash, newname, newsize);
+          fle->file = NULL;
+          gftp_file_destroy (fle);
+        }
+      gftp_end_transfer (request);
+      g_free (fle);
+    }
+  else
+    {
+      g_hash_table_destroy (dirhash);
+      dirhash = NULL;
+    }
+  return (dirhash);
+}
+
+
+static void
+destroy_hash_ent (gpointer key, gpointer value, gpointer user_data)
+{
+
+  g_free (key);
+  g_free (value);
+}
+
+
+static void
+gftp_destroy_dir_hash (GHashTable * dirhash)
+{
+  g_hash_table_foreach (dirhash, destroy_hash_ent, NULL);
+  g_hash_table_destroy (dirhash);
 }
 
 
@@ -1574,57 +1581,6 @@ gftp_get_all_subdirs (gftp_transfer * transfer,
 }
 
 
-static GHashTable *
-gftp_gen_dir_hash (gftp_request * request)
-{
-  unsigned long *newsize;
-  GHashTable * dirhash;
-  gftp_file * fle;
-  char * newname;
-
-
-  dirhash = g_hash_table_new (string_hash_function, string_hash_compare);
-  if (gftp_list_files (request) == 0)
-    {
-      fle = g_malloc0 (sizeof (*fle));
-      while (gftp_get_next_file (request, NULL, fle) > 0)
-        {
-          newname = fle->file;
-          newsize = g_malloc (sizeof (unsigned long));
-          *newsize = fle->size;
-          g_hash_table_insert (dirhash, newname, newsize);
-          fle->file = NULL;
-          gftp_file_destroy (fle);
-        }
-      gftp_end_transfer (request);
-      g_free (fle);
-    }
-  else
-    {
-      g_hash_table_destroy (dirhash);
-      dirhash = NULL;
-    }
-  return (dirhash);
-}
-
-
-static void
-gftp_destroy_dir_hash (GHashTable * dirhash)
-{
-  g_hash_table_foreach (dirhash, destroy_hash_ent, NULL);
-  g_hash_table_destroy (dirhash);
-}
-
-
-static void
-destroy_hash_ent (gpointer key, gpointer value, gpointer user_data)
-{
-
-  g_free (key);
-  g_free (value);
-}
-
-
 int
 gftp_get_file_transfer_mode (char *filename, int def)
 {
@@ -1724,6 +1680,9 @@ gftp_connect_server (gftp_request * request, char *service)
     {
       disphost = res->ai_canonname ? res->ai_canonname : connect_host;
       port = get_port (res);
+      if (!request->use_proxy)
+        request->port = port;
+
       if ((sock = socket (res->ai_family, res->ai_socktype, 
                           res->ai_protocol)) < 0)
         {
@@ -1731,7 +1690,7 @@ gftp_connect_server (gftp_request * request, char *service)
                                      _("Failed to create a socket: %s\n"),
                                      g_strerror (errno));
           continue; 
-        }
+        } 
 
       request->logging_function (gftp_logging_misc, request->user_data,
 				 _("Trying %s:%d\n"), disphost, port);
@@ -1794,6 +1753,9 @@ gftp_connect_server (gftp_request * request, char *service)
           port = serv_struct.s_port;
           request->port = ntohs (serv_struct.s_port);
         }
+
+      if (!request->use_proxy)
+        request->port = ntohs (port);
     }
   remote_address.sin_port = port;
 
@@ -1840,16 +1802,15 @@ gftp_connect_server (gftp_request * request, char *service)
   port = ntohs (port);
 #endif /* HAVE_GETADDRINFO */
 
-  if (!request->use_proxy && disphost != NULL)
-    {
-      g_free (request->hostname);
-      request->hostname = g_malloc (strlen (disphost) + 1);
-      strcpy (request->hostname, disphost);
-      connect_host = request->hostname;
-    }
-
   request->logging_function (gftp_logging_misc, request->user_data,
 			     _("Connected to %s:%d\n"), connect_host, port);
+
+  if (gftp_set_sockblocking (request, sock, 1) == -1)
+    {
+      close (sock);
+      return (-1);
+    }
+
   return (sock);
 }
 
@@ -1857,56 +1818,13 @@ gftp_connect_server (gftp_request * request, char *service)
 void
 gftp_set_config_options (gftp_request * request)
 {
-  request->transfer_type = passive_transfer ? gftp_transfer_passive : gftp_transfer_active;
   request->network_timeout = network_timeout;
   request->retries = retries;
   request->sleep_time = sleep_time;
   request->maxkbs = maxkbs;
 
-  if (request->protonum == GFTP_SSHV2_NUM || 
-      request->protonum == GFTP_SSH_NUM)
-    request->need_userpass = ssh_need_userpass;
-
-  /* Set up the proxy server settings. If we are using a HTTP proxy for FTP,
-     then set up the HTTP proxy settings */
-  if ((strcmp (request->protocol_name, "FTP") == 0 ||
-       strcmp (request->protocol_name, "HTTP") == 0) &&
-      proxy_config != NULL)
-    {
-      if (strcmp (request->protocol_name, "FTP") == 0 &&
-          strcmp (proxy_config, "http") != 0)
-        {
-          gftp_set_proxy_hostname (request, firewall_host);
-          gftp_set_proxy_port (request, firewall_port);
-          gftp_set_proxy_username (request, firewall_username);
-          gftp_set_proxy_password (request, firewall_password);
-          gftp_set_proxy_account (request, firewall_account);
-          gftp_set_proxy_config (request, proxy_config);
-        }
-      else 
-        {
-          gftp_set_proxy_hostname (request, http_proxy_host);
-          gftp_set_proxy_port (request, http_proxy_port);
-          gftp_set_proxy_username (request, http_proxy_username);
-          gftp_set_proxy_password (request, http_proxy_password);
-
-          
-          if (request->proxy_config == NULL)
-            {
-              if (strcmp (GFTP_GET_PROTOCOL_NAME (request), "FTP") == 0)
-                {
-                  gftp_protocols[GFTP_HTTP_NUM].init (request);
-                  request->proxy_config = g_malloc (4);
-                  strcpy (request->proxy_config, "ftp");
-                }
-              else
-                {
-                  request->proxy_config = g_malloc (5);
-                  strcpy (request->proxy_config, "http");
-                }
-            }
-        }
-    }
+  if (request->set_config_options != NULL)
+    request->set_config_options (request);
 }
 
 
@@ -1938,9 +1856,14 @@ print_file_list (GList * list)
   for (templist = list; ; templist = templist->next)
     {
       tempfle = templist->data;
-      printf ("%s:%s:%ld:%ld:%s:%s:%s\n", tempfle->file, tempfle->destfile,
-              tempfle->size, tempfle->startsize, tempfle->user, tempfle->group,
-              tempfle->attribs);
+#if defined (_LARGEFILE_SOURCE)
+      printf ("%s:%s:%lld:%lld:%s:%s:%s\n", 
+#else
+      printf ("%s:%s:%ld:%ld:%s:%s:%s\n", 
+#endif
+              tempfle->file, tempfle->destfile, 
+              tempfle->size, tempfle->startsize, 
+              tempfle->user, tempfle->group, tempfle->attribs);
       if (templist->next == NULL)
         break;
     }
@@ -1949,12 +1872,328 @@ print_file_list (GList * list)
   for (; ; templist = templist->prev)
     {
       tempfle = templist->data;
-      printf ("%s:%s:%ld:%ld:%s:%s:%s\n", tempfle->file, tempfle->destfile,
-              tempfle->size, tempfle->startsize, tempfle->user, tempfle->group,
-              tempfle->attribs);
+#if defined (_LARGEFILE_SOURCE)
+      printf ("%s:%s:%lld:%lld:%s:%s:%s\n", 
+#else
+      printf ("%s:%s:%ld:%ld:%s:%s:%s\n", 
+#endif
+              tempfle->file, tempfle->destfile, 
+              tempfle->size, tempfle->startsize, 
+              tempfle->user, tempfle->group, tempfle->attribs);
       if (templist == list)
         break;
     }
   printf ("--END OF FILE LISTING--\n");
 }
 
+
+static void
+gftp_free_getline_buffer (gftp_getline_buffer ** rbuf)
+{
+  g_free ((*rbuf)->buffer);
+  g_free (*rbuf);
+  *rbuf = NULL;
+}
+
+
+ssize_t
+gftp_get_line (gftp_request * request, gftp_getline_buffer ** rbuf, 
+               char * str, size_t len, int fd)
+{
+  ssize_t ret, retval, rlen, copysize;
+  char *pos, *nextpos;
+
+  if (*rbuf == NULL)
+    {
+      *rbuf = g_malloc0 (sizeof (**rbuf));
+      (*rbuf)->max_bufsize = len;
+      (*rbuf)->buffer = g_malloc ((*rbuf)->max_bufsize + 1);
+      if ((ret = gftp_read (request, (*rbuf)->buffer, (*rbuf)->max_bufsize, 
+                            fd)) <= 0)
+        {
+          gftp_free_getline_buffer (rbuf);
+          return (ret);
+        }
+      (*rbuf)->buffer[ret] = '\0';
+      (*rbuf)->cur_bufsize = ret;
+      (*rbuf)->curpos = (*rbuf)->buffer;
+    }
+
+  retval = -2;
+
+  do
+    {
+      if ((*rbuf)->cur_bufsize > 0 &&
+          ((pos = strchr ((*rbuf)->curpos, '\n')) != NULL ||
+           ((*rbuf)->curpos == (*rbuf)->buffer && 
+            (*rbuf)->max_bufsize == (*rbuf)->cur_bufsize)))
+        {
+          if (pos != NULL)
+            {
+              nextpos = pos + 1;
+              if (pos > (*rbuf)->curpos && *(pos - 1) == '\r')
+                pos--;
+              *pos = '\0';
+            }
+          else
+            nextpos = NULL;
+
+          strncpy (str, (*rbuf)->curpos, len);
+          retval = pos - (*rbuf)->curpos > len ? len : pos - (*rbuf)->curpos;
+
+          if (pos != NULL)
+            {
+              if (nextpos - (*rbuf)->buffer >= (*rbuf)->cur_bufsize)
+                (*rbuf)->cur_bufsize = 0;
+              else
+                (*rbuf)->curpos = nextpos;
+            }
+
+          break;
+        }
+      else
+        {
+          if ((*rbuf)->cur_bufsize == 0 || *(*rbuf)->curpos == '\0')
+            {
+              rlen = (*rbuf)->max_bufsize;
+              pos = (*rbuf)->buffer;
+              copysize = 0;
+            }
+          else
+            {
+              copysize = (*rbuf)->cur_bufsize - ((*rbuf)->curpos - (*rbuf)->buffer);
+              memmove ((*rbuf)->buffer, (*rbuf)->curpos, copysize);
+              pos = (*rbuf)->buffer + copysize;
+              rlen = (*rbuf)->max_bufsize - copysize;
+            }
+          (*rbuf)->curpos = (*rbuf)->buffer;
+
+          if ((ret = gftp_read (request, pos, rlen, fd)) <= 0)
+            {
+              gftp_free_getline_buffer (rbuf);
+              return (ret);
+            }
+          (*rbuf)->buffer[ret + copysize] = '\0';
+          (*rbuf)->cur_bufsize = ret + copysize;
+        }
+    }
+  while (retval == -2);
+
+  return (retval);
+}
+
+
+ssize_t 
+gftp_read (gftp_request * request, void *ptr, size_t size, int fd)
+{
+  struct timeval tv;
+  fd_set fset;
+  ssize_t ret;
+
+  errno = 0;
+  ret = 0;
+  do
+    {
+      FD_ZERO (&fset);
+      FD_SET (fd, &fset);
+      if (request != NULL)
+        tv.tv_sec = request->network_timeout;
+      else
+        tv.tv_sec = network_timeout;
+      tv.tv_usec = 0;
+      ret = select (fd + 1, &fset, NULL, NULL, &tv);
+      if (ret == -1 && errno == EINTR)
+        {
+          if (request && request->cancel)
+            break;
+          else
+            continue;
+        }
+      else if (ret <= 0)
+        {
+          if (request != NULL)
+            {
+              request->logging_function (gftp_logging_error, request->user_data,
+                                         _("Connection to %s timed out\n"),
+                                         request->hostname);
+              gftp_disconnect (request);
+            }
+          return (-1);
+        }
+
+      if ((ret = read (fd, ptr, size)) < 0)
+        {
+          if (errno == EINTR)
+            {
+              if (request != NULL && request->cancel)
+                break;
+              else
+                continue;
+             }
+ 
+          if (request != NULL)
+            {
+              request->logging_function (gftp_logging_error, request->user_data,
+                                   _("Error: Could not read from socket: %s\n"),
+                                    g_strerror (errno));
+              gftp_disconnect (request);
+            }
+          return (-1);
+        }
+    }
+  while (errno == EINTR && !(request != NULL && request->cancel));
+
+  if (errno == EINTR && request != NULL && request->cancel)
+    {
+      gftp_disconnect (request);
+      return (-1);
+    }
+
+  return (ret);
+}
+
+
+ssize_t 
+gftp_write (gftp_request * request, const char *ptr, size_t size, int fd)
+{
+  struct timeval tv;
+  size_t ret, w_ret;
+  fd_set fset;
+
+  errno = 0;
+  ret = 0;
+  do
+    {
+      FD_ZERO (&fset);
+      FD_SET (fd, &fset);
+      if (request != NULL)
+        tv.tv_sec = request->network_timeout;
+      else
+        tv.tv_sec = network_timeout;
+      tv.tv_usec = 0;
+      ret = select (fd + 1, NULL, &fset, NULL, &tv);
+      if (ret == -1 && errno == EINTR)
+        {
+          if (request != NULL && request->cancel)
+            break;
+          else
+            continue;
+        }
+      else if (ret <= 0)
+        {
+          if (request != NULL)
+            {
+              request->logging_function (gftp_logging_error, request->user_data,
+                                         _("Connection to %s timed out\n"),
+                                         request->hostname);
+              gftp_disconnect (request);
+            }
+          return (-1);
+        }
+
+      if ((w_ret = write (fd, ptr, size)) < 0)
+        {
+          if (errno == EINTR)
+            {
+              if (request != NULL && request->cancel)
+                break;
+              else
+                continue;
+             }
+ 
+          if (request != NULL)
+            {
+              request->logging_function (gftp_logging_error, request->user_data,
+                                    _("Error: Could not write to socket: %s\n"),
+                                    g_strerror (errno));
+              gftp_disconnect (request);
+            }
+          return (-1);
+        }
+
+      ptr += w_ret;
+      size -= w_ret;
+      ret += w_ret;
+    }
+  while (size > 0);
+
+  if (errno == EINTR && request != NULL && request->cancel)
+    {
+      gftp_disconnect (request);
+      return (-1);
+    }
+
+  return (ret);
+}
+
+
+ssize_t 
+gftp_writefmt (gftp_request * request, int fd, const char *fmt, ...)
+{
+  char *tempstr;
+  va_list argp;
+  ssize_t ret;
+
+  va_start (argp, fmt);
+  tempstr = g_strdup_vprintf (fmt, argp);
+  va_end (argp);
+
+  ret = gftp_write (request, tempstr, strlen (tempstr), fd);
+  g_free (tempstr);
+  return (ret);
+}
+
+
+int
+gftp_set_sockblocking (gftp_request * request, int fd, int non_blocking)
+{
+  int flags;
+
+  if ((flags = fcntl (fd, F_GETFL, 0)) == -1)
+    {
+      request->logging_function (gftp_logging_error, request->user_data,
+                                 _("Cannot get socket flags: %s\n"),
+                                 g_strerror (errno));
+      gftp_disconnect (request);
+      return (-1);
+    }
+
+  if (non_blocking)
+    flags |= O_NONBLOCK;
+  else
+    flags &= ~O_NONBLOCK;
+
+  if (fcntl (fd, F_SETFL, flags) == -1)
+    {
+      request->logging_function (gftp_logging_error, request->user_data,
+                                 _("Cannot set socket to non-blocking: %s\n"),
+                                 g_strerror (errno));
+      gftp_disconnect (request);
+      return (-1);
+    }
+
+  return (0);
+}
+
+
+void
+gftp_swap_socks (gftp_request * dest, gftp_request * source)
+{
+  g_return_if_fail (dest != NULL);
+  g_return_if_fail (source != NULL);
+  g_return_if_fail (dest->protonum == source->protonum);
+
+  dest->sockfd = source->sockfd;
+  dest->datafd = source->datafd;
+  dest->cached = 0;
+  if (!source->always_connected)
+    {
+      source->sockfd = -1;
+      source->datafd = -1;
+      source->cached = 1;
+    }
+
+  if (dest->swap_socks)
+    dest->swap_socks (dest, source);
+}
+
