@@ -18,10 +18,87 @@
 /*****************************************************************************/
 
 #include "gftp-gtk.h"
+static const char cvsid[] = "$Id: dnd.c,v 1.9 2002/11/27 14:29:55 masneyb Exp $";
+
+
+static int
+dnd_remote_file (char *url, gftp_window_data * wdata)
+{
+  gftp_window_data * other_wdata, * fromwdata;
+  gftp_request * current_ftpdata;
+  gftp_file * newfle;
+  GList * templist;
+  char *pos;
+
+  if (wdata == &window1)
+    other_wdata = &window2;
+  else if (wdata == &window2)
+    other_wdata = &window1;
+  else 
+    other_wdata = NULL;
+    
+  newfle = g_malloc0 (sizeof (*newfle));
+  newfle->shown = 1;
+  if (url[strlen (url) - 1] == '/') 
+    {
+      newfle->isdir = 1;
+      url[strlen (url) - 1] = '\0';
+    }
+
+  current_ftpdata = gftp_request_new ();
+  current_ftpdata->logging_function = ftp_log;
+
+  if (gftp_parse_url (current_ftpdata, url) != 0 ||
+      current_ftpdata->directory == NULL ||
+      (pos = strrchr (current_ftpdata->directory, '/')) == NULL) 
+    {
+      ftp_log (gftp_logging_misc, NULL, 
+               _("Drag-N-Drop: Ignoring url %s: Not a valid url\n"), url);
+      gftp_request_destroy (current_ftpdata, 1);
+      free_fdata (newfle);
+      return (0);
+    }
+
+  *pos++ = '\0';
+  if (compare_request (current_ftpdata, wdata->request, 1))
+    {
+      gftp_request_destroy (current_ftpdata, 1);
+      return (0);
+    }
+
+  if (other_wdata != NULL && 
+      compare_request (current_ftpdata, other_wdata->request, 1))
+    {
+      if (other_wdata->request->password != NULL)
+        gftp_set_password (current_ftpdata, other_wdata->request->password);
+
+      fromwdata = other_wdata;
+    }
+  else
+    fromwdata = NULL;
+
+  *(pos - 1) = '/';
+  newfle->file = g_malloc (strlen (current_ftpdata->directory) + 1);
+  strcpy (newfle->file, current_ftpdata->directory);
+  *(pos - 1) = '\0';
+  
+  newfle->destfile = g_strconcat (GFTP_GET_DIRECTORY (wdata->request),
+                                     "/", pos, NULL);
+  newfle->ascii = gftp_get_file_transfer_mode (newfle->file, 
+                                wdata->request->data_type) == GFTP_TYPE_ASCII;
+
+  templist = g_malloc0 (sizeof (*templist));
+  templist->data = newfle;
+  templist->next = NULL;
+  add_file_transfer (current_ftpdata, wdata->request, fromwdata,
+                     wdata, templist, 1);
+  gftp_request_destroy (current_ftpdata, 1);
+
+  /* FIXME  resume files and download entire directories */
+
+  return (1);
+}
 
-static int dnd_remote_file 			( char *url, 
-						  GList ** transfers, 
-						  gftp_window_data * wdata );
 
 void
 openurl_get_drag_data (GtkWidget * widget, GdkDragContext * context, gint x,
@@ -77,38 +154,48 @@ listbox_drag (GtkWidget * widget, GdkDragContext * context,
         continue;
 
       oldlen = totlen;
-      if (GFTP_GET_HOSTNAME (wdata->request) == NULL)
+      if (GFTP_GET_HOSTNAME (wdata->request) == NULL || 
+          wdata->request->protonum == GFTP_LOCAL_NUM)
         {
-          tempstr = g_strdup_printf ("%s://%s/%s%c", 
+          tempstr = g_strdup_printf ("%s://%s/%s ", 
                                  GFTP_GET_URL_PREFIX (wdata->request),
                                  GFTP_GET_DIRECTORY (wdata->request), 
-                                 tempfle->file, tempfle->isdir ? '/' : '\0');
+                                 tempfle->file);
         }
       else if (GFTP_GET_USERNAME (wdata->request) == NULL 
                || *GFTP_GET_USERNAME (wdata->request) == '\0')
         {
-          tempstr = g_strdup_printf ("%s://%s:%d%s/%s%c", 
+          tempstr = g_strdup_printf ("%s://%s:%d%s/%s ", 
                                  GFTP_GET_URL_PREFIX (wdata->request),
                                  GFTP_GET_HOSTNAME (wdata->request),
                                  GFTP_GET_PORT (wdata->request),
                                  GFTP_GET_DIRECTORY (wdata->request), 
-                                 tempfle->file, tempfle->isdir ? '/' : '\0');
+                                 tempfle->file);
         }
       else
         {
-          tempstr = g_strdup_printf ("%s://%s@%s:%d%s/%s%c", 
+          tempstr = g_strdup_printf ("%s://%s@%s:%d%s/%s ", 
                                  GFTP_GET_URL_PREFIX (wdata->request),
                                  GFTP_GET_USERNAME (wdata->request), 
                                  GFTP_GET_HOSTNAME (wdata->request),
                                  GFTP_GET_PORT (wdata->request),
                                  GFTP_GET_DIRECTORY (wdata->request), 
-                                 tempfle->file, tempfle->isdir ? '/' : '\0');
+                                 tempfle->file);
         }
+
       if ((pos = strchr (tempstr, ':')) != NULL)
-        pos += 2;
+        pos += 3;
       else
         pos = tempstr;
       remove_double_slashes (pos);
+
+      /* Note, I am allocating memory for this byte above. Note the extra space
+         at the end of the g_strdup_printf() format argument */
+      if (tempfle->isdir)
+        tempstr[strlen (tempstr) - 1] = '/';
+      else
+        tempstr[strlen (tempstr) - 1] = '\0';
+
       totlen += strlen (tempstr);
       if (str != NULL)
         {
@@ -139,28 +226,25 @@ listbox_get_drag_data (GtkWidget * widget, GdkDragContext * context, gint x,
 		       gint y, GtkSelectionData * selection_data, guint info,
 		       guint32 clk_time, gpointer data)
 {
-  GList * new_file_transfers, * templist;
   char *newpos, *oldpos, tempchar;
   gftp_window_data * wdata;
-  gftp_transfer * tdata;
   int finish_drag;
 
   wdata = data;   
   if (!check_status (_("Drag-N-Drop"), wdata, 1, 0, 0, 1)) 
     return;
 
-  new_file_transfers = NULL;
   finish_drag = 0;
   if ((selection_data->length >= 0) && (selection_data->format == 8)) 
     {
       oldpos = (char *) selection_data->data;
-      while ((newpos = strchr (oldpos, '\n')) 
-             || (newpos = strchr (oldpos, '\0'))) 
+      while ((newpos = strchr (oldpos, '\n')) || 
+             (newpos = strchr (oldpos, '\0'))) 
         {
           tempchar = *newpos;
           *newpos = '\0';
           ftp_log (gftp_logging_misc, NULL, _("Received URL %s\n"), oldpos);
-          if (dnd_remote_file (oldpos, &new_file_transfers, wdata))
+          if (dnd_remote_file (oldpos, wdata))
             finish_drag = 1;
          
           if (*newpos == '\0') 
@@ -169,131 +253,6 @@ listbox_get_drag_data (GtkWidget * widget, GdkDragContext * context, gint x,
         }
     }
 
-  if (finish_drag) 
-    {
-      for (templist = new_file_transfers; templist != NULL; 
-           templist = templist->next)
-        {
-          tdata = templist->data;
-	  if (tdata->files != NULL)
-            add_file_transfer (tdata->fromreq, tdata->toreq, tdata->fromwdata,
-                               tdata->towdata, tdata->files, 0);
-        }
-    }
   gtk_drag_finish (context, finish_drag, FALSE, clk_time);
 }
 
-
-static int
-dnd_remote_file (char *url, GList ** transfers, gftp_window_data * wdata)
-{
-  gftp_request * current_ftpdata;
-  gftp_window_data * fromwdata;
-  gftp_transfer * tdata;
-  gftp_file * newfle;
-  GList * templist;
-  char *str, *pos;
-  int i;
-
-  newfle = g_malloc0 (sizeof (*newfle));
-  newfle->shown = 1;
-  if (url[strlen (url) - 1] == '/') 
-    {
-      newfle->isdir = 1;
-      url[strlen (url) - 1] = '\0';
-    }
-
-  current_ftpdata = gftp_request_new ();
-  current_ftpdata->logging_function = ftp_log;
-
-  if (gftp_parse_url (current_ftpdata, url) != 0) 
-    {
-      ftp_log (gftp_logging_misc, NULL, 
-               _("Drag-N-Drop: Ignoring url %s: Not a valid url\n"), url);
-      gftp_request_destroy (current_ftpdata);
-      free_fdata (newfle);
-      return (0);
-    }
-
-  if ((str = GFTP_GET_DIRECTORY (current_ftpdata)) != NULL) 
-    {
-      if ((pos = strrchr (str, '/')) == NULL) 
-        pos = str;
-      else pos++;
-      *(pos - 1) = '\0';
-      i = 1;
-    }
-  else 
-    {
-      pos = str = GFTP_GET_DIRECTORY (current_ftpdata);
-      i = 0;
-    }
-
-  if (compare_request (current_ftpdata, wdata->request, 1))
-    return (0);
-
-  if (i)
-    {
-      *(pos - 1) = '/';
-      newfle->file = g_malloc (strlen (str) + 1);
-      strcpy (newfle->file, str);
-      *(pos - 1) = '\0';
-    }
-  else
-    {
-      newfle->file = g_malloc (strlen (str) + 1);
-      strcpy (newfle->file, str);
-    }
-  
-  newfle->destfile = g_strconcat (GFTP_GET_DIRECTORY (wdata->request),
-                                     "/", pos, NULL);
-  newfle->ascii = gftp_get_file_transfer_mode (newfle->file, 
-                                wdata->request->data_type) == GFTP_TYPE_ASCII;
-
-  tdata = NULL;
-  templist = *transfers;
-  while (templist != NULL) 
-    {
-      tdata = templist->data;
-      if (compare_request (tdata->fromreq, current_ftpdata, 1))
-        break;
-      templist = templist->next;
-    }
-
-  if (tdata == NULL) 
-    {
-      tdata = g_malloc0 (sizeof (*tdata));
-      tdata->towdata = wdata == &window1 ? &window1 : &window2;
-      fromwdata = wdata == &window1 ? &window2 : &window1;
-      if (compare_request (fromwdata->request, current_ftpdata, 1))
-        {
-          gftp_set_password (current_ftpdata, fromwdata->request->password);
-          tdata->fromwdata = fromwdata;
-        }
-      tdata->fromreq = current_ftpdata;
-      tdata->toreq = gftp_request_new ();
-      tdata->toreq->logging_function = ftp_log;
-      tdata->toreq = copy_request (wdata->request);
-      *transfers = g_list_append (*transfers, tdata);
-    }
-  else
-    gftp_request_destroy (current_ftpdata);
-
-  if (newfle->isdir)
-    {
-/* FIXME - need to fix this
-      add_entire_directory (tdata, newfle, 
-		            GFTP_GET_DIRECTORY (tdata->fromhdata->ftpdata), 
-			    GFTP_GET_DIRECTORY (tdata->tohdata->ftpdata), 
-			    tdata->fromhdata->ftpdata);
-*/
-    }
-  else
-    {
-      tdata->files = g_list_append (tdata->files, newfle);
-      if (tdata->curfle == NULL) 
-	tdata->curfle = tdata->files;
-    }
-  return (1);
-}
-
