@@ -18,27 +18,12 @@
 /*****************************************************************************/
 
 #include <gftp-gtk.h>
+static const char cvsid[] = "$Id: misc-gtk.c,v 1.18 2002/11/27 02:23:51 masneyb Exp $";
 
-static void set_menu_sensitive 			( gftp_window_data * wdata,
-						  char *path, 
-						  int sensitive );
-static void destroy_dialog_data 		( gftp_dialog_data * data );
-static gint delete_event 			( GtkWidget * widget, 
-						  GdkEvent * event, 
-						  gpointer data );
-static void trans_stop_button 			( GtkWidget * widget,
-						  gpointer data );
-
+static pthread_mutex_t netfunclock = PTHREAD_MUTEX_INITIALIZER;
 static GtkWidget * statuswid;
 
 
-void
-fix_display (void)
-{
-  while (g_main_iteration (FALSE));
-}
-
-
 void
 remove_files_window (gftp_window_data * wdata)
 {
@@ -54,23 +39,24 @@ remove_files_window (gftp_window_data * wdata)
 void
 ftp_log (gftp_logging_level level, void *ptr, const char *string, ...)
 {
-  char tempstr[512];
+  int upd, free_logstr;
   gftp_log * newlog;
+  char *logstr;
   gint delsize;
   va_list argp;
   guint len;
-  int upd;
-#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
+#if GTK_MAJOR_VERSION == 1
   gftp_color * color;
   GdkColor fore;
 #else
   GtkTextBuffer * textbuf;
   GtkTextIter iter, iter2;
+  gsize bread, bwrite;
   const char *descr;
-  guint inslen;
+  char *tempstr;
 #endif
 
-  if (ptr == (void *) 0x1)
+  if (pthread_self () != main_thread_id)
     {
       newlog = g_malloc0 (sizeof (*newlog));
       newlog->type = level;
@@ -85,23 +71,39 @@ ftp_log (gftp_logging_level level, void *ptr, const char *string, ...)
     }
 
   va_start (argp, string);
-  g_vsnprintf (tempstr, sizeof (tempstr), string, argp);
+  if (strcmp (string, "%s") == 0)
+    {
+      logstr = va_arg (argp, char *);
+      free_logstr = 0;
+    }
+  else
+    {
+      logstr = g_strdup_vprintf (string, argp);
+      free_logstr = 1;
+    }
   va_end (argp);
 
   if (logfd != NULL)
     {
-      if (fwrite (tempstr, strlen (tempstr), 1, logfd) != 1)
+      if (fwrite (logstr, strlen (logstr), 1, logfd) != 1)
         {
           fclose (logfd);
           logfd = NULL;
         }
       else
-        fflush (logfd);
+        {
+          fflush (logfd);
+          if (ferror (logfd))
+            {
+              fclose (logfd);
+              logfd = NULL;
+            }
+        }
     }
 
   upd = logwdw_vadj->upper - logwdw_vadj->page_size == logwdw_vadj->value;
 
-#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
+#if GTK_MAJOR_VERSION == 1
   switch (level)
     {
       case gftp_logging_send:
@@ -124,7 +126,7 @@ ftp_log (gftp_logging_level level, void *ptr, const char *string, ...)
   fore.blue = color->blue;
 
   gtk_text_freeze (GTK_TEXT (logwdw));
-  gtk_text_insert (GTK_TEXT (logwdw), NULL, &fore, NULL, tempstr, -1);
+  gtk_text_insert (GTK_TEXT (logwdw), NULL, &fore, NULL, logstr, -1);
 
   len = gtk_text_get_length (GTK_TEXT (logwdw));
   if (max_log_window_size > 0 && len > max_log_window_size)
@@ -157,11 +159,24 @@ ftp_log (gftp_logging_level level, void *ptr, const char *string, ...)
         break;
     }
 
+  /* If the current log message is not in UTF8 format, convert it to UTF8 
+     format based on the current locale */
+  if (!g_utf8_validate (logstr, -1, NULL))
+    {
+      tempstr = g_locale_to_utf8 (logstr, -1, &bread, &bwrite, NULL);
+      if (tempstr != NULL)
+        {
+          if (free_logstr)
+            g_free (logstr);
+          logstr = tempstr;
+          free_logstr = 1;
+        }
+    }
+
   textbuf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (logwdw));
   len = gtk_text_buffer_get_char_count (textbuf);
-  inslen = strlen (tempstr);
   gtk_text_buffer_get_iter_at_offset (textbuf, &iter, len);
-  gtk_text_buffer_insert_with_tags_by_name (textbuf, &iter, tempstr, inslen, 
+  gtk_text_buffer_insert_with_tags_by_name (textbuf, &iter, logstr, -1, 
                                             descr, NULL);
 
   if (upd)
@@ -171,17 +186,21 @@ ftp_log (gftp_logging_level level, void *ptr, const char *string, ...)
                                      0, 1, 1, 1);
     }
 
-  delsize = len + inslen - max_log_window_size;
-  if (max_log_window_size > 0 && delsize > 0)
+  if (max_log_window_size > 0)
     {
-      gtk_text_buffer_get_iter_at_offset (textbuf, &iter, 0);
-      gtk_text_buffer_get_iter_at_offset (textbuf, &iter2, delsize);
-      gtk_text_buffer_delete (textbuf, &iter, &iter2);
+      delsize = len + g_utf8_strlen (logstr, -1) - max_log_window_size;
+
+      if (delsize > 0)
+        {
+          gtk_text_buffer_get_iter_at_offset (textbuf, &iter, 0);
+          gtk_text_buffer_get_iter_at_offset (textbuf, &iter2, delsize);
+          gtk_text_buffer_delete (textbuf, &iter, &iter2);
+        }
     }
 #endif
 
-  if (ptr == NULL)
-    fix_display ();
+  if (free_logstr)
+    g_free (logstr);
 }
 
 
@@ -192,10 +211,9 @@ refresh (gftp_window_data * wdata)
     return;
   gtk_clist_freeze (GTK_CLIST (wdata->listbox));
   remove_files_window (wdata);
-  gftp_delete_cache_entry (wdata->request);
+  gftp_delete_cache_entry (wdata->request, 0);
   ftp_list_files (wdata, 0);
   gtk_clist_thaw (GTK_CLIST (wdata->listbox));
-  fix_display ();
 }
 
 
@@ -240,12 +258,35 @@ update_window_info (void)
   update_window (&window1);
   update_window (&window2);
 
-  tempwid = gtk_item_factory_get_widget (factory, _(menus[tools_start+2].path));
+  tempwid = gtk_item_factory_get_widget (factory, menus[tools_start+2].path);
   gtk_widget_set_sensitive (tempwid, GFTP_IS_CONNECTED (window1.request) 
 			    && GFTP_IS_CONNECTED (window2.request));
 }
 
 
+static void
+set_menu_sensitive (gftp_window_data * wdata, char *path, int sensitive)
+{
+  GtkWidget * tempwid;
+  char * pos;
+
+  tempwid = NULL;
+
+  if (factory != NULL)
+    tempwid = gtk_item_factory_get_widget (factory, path);
+  if (tempwid)
+    gtk_widget_set_sensitive (tempwid, sensitive);
+
+  if ((pos = strchr (path + 1, '/')) == NULL)
+    pos = path;
+
+  if (wdata->ifactory)
+    tempwid = gtk_item_factory_get_widget (wdata->ifactory, pos);
+  if (tempwid)
+    gtk_widget_set_sensitive (tempwid, sensitive);
+}
+
+
 void
 update_window (gftp_window_data * wdata)
 {
@@ -257,7 +298,8 @@ update_window (gftp_window_data * wdata)
     {
       fspec = wdata->show_selected ? "Selected" : strcmp (wdata->filespec, "*") == 0 ?  _("All Files") : wdata->filespec;
 
-      if ((temp1str = GFTP_GET_HOSTNAME (wdata->request)) == NULL)
+      if ((temp1str = GFTP_GET_HOSTNAME (wdata->request)) == NULL ||
+          wdata->request->protonum == GFTP_LOCAL_NUM)
 	temp1str = "";
       tempstr = g_strconcat (temp1str, *temp1str == '\0' ? "[" : " [",
 		     GFTP_GET_PROTOCOL_NAME (wdata->request),
@@ -294,51 +336,28 @@ update_window (gftp_window_data * wdata)
   set_menu_sensitive (wdata, menus[start + 7].path, connected);
   set_menu_sensitive (wdata, menus[start + 8].path, connected);
   set_menu_sensitive (wdata, menus[start + 9].path, connected);
-  set_menu_sensitive (wdata, menus[start + 11].path, connected &&
-                      wdata->request->site != NULL);
+  set_menu_sensitive (wdata, menus[start + 11].path, connected);
   set_menu_sensitive (wdata, menus[start + 12].path, connected &&
-                      wdata->request->chdir!= NULL);
+                      wdata->request->site != NULL);
   set_menu_sensitive (wdata, menus[start + 13].path, connected &&
-                      wdata->request->chmod != NULL);
+                      wdata->request->chdir!= NULL);
   set_menu_sensitive (wdata, menus[start + 14].path, connected &&
-                      wdata->request->mkdir != NULL);
+                      wdata->request->chmod != NULL);
   set_menu_sensitive (wdata, menus[start + 15].path, connected &&
-                      wdata->request->rename != NULL);
+                      wdata->request->mkdir != NULL);
   set_menu_sensitive (wdata, menus[start + 16].path, connected &&
+                      wdata->request->rename != NULL);
+  set_menu_sensitive (wdata, menus[start + 17].path, connected &&
                       wdata->request->rmdir != NULL &&
                       wdata->request->rmfile != NULL);
-  set_menu_sensitive (wdata, menus[start + 17].path, connected &&
-                      wdata->request->get_file != NULL);
   set_menu_sensitive (wdata, menus[start + 18].path, connected &&
                       wdata->request->get_file != NULL);
-  set_menu_sensitive (wdata, menus[start + 19].path, connected);
-  fix_display ();
+  set_menu_sensitive (wdata, menus[start + 19].path, connected &&
+                      wdata->request->get_file != NULL);
+  set_menu_sensitive (wdata, menus[start + 20].path, connected);
 }  
 
 
-static void
-set_menu_sensitive (gftp_window_data * wdata, char *path, int sensitive)
-{
-  GtkWidget * tempwid;
-  char * pos;
-
-  tempwid = NULL;
-
-  if (factory != NULL)
-    tempwid = gtk_item_factory_get_widget (factory, _(path));
-  if (tempwid)
-    gtk_widget_set_sensitive (tempwid, sensitive);
-
-  if ((pos = strchr (path + 1, '/')) == NULL)
-    pos = path;
-
-  if (wdata->ifactory)
-    tempwid = gtk_item_factory_get_widget (wdata->ifactory, _(pos));
-  if (tempwid)
-    gtk_widget_set_sensitive (tempwid, sensitive);
-}
-
-
 GtkWidget *
 toolbar_pixmap (GtkWidget * widget, char *filename)
 {
@@ -405,12 +424,12 @@ gftp_free_pixmap (char *filename)
   if ((graphic = g_hash_table_lookup (graphic_hash_table, filename)) == NULL)
     return;
 
-#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
+#if GTK_MAJOR_VERSION == 1
   gdk_pixmap_unref (graphic->pixmap);
   gdk_bitmap_unref (graphic->bitmap);
 #else
-  gdk_drawable_unref (graphic->pixmap);
-  gdk_drawable_unref (graphic->bitmap);
+  g_object_unref (graphic->pixmap);
+  g_object_unref (graphic->bitmap);
 #endif
 
   g_hash_table_remove (graphic_hash_table, filename);
@@ -503,24 +522,67 @@ check_status (char *name, gftp_window_data *wdata, int check_other_stop,
 }
 
 
+static gchar *
+gftp_item_factory_translate (const char *path,
+			     gpointer    func_data)
+{
+  const gchar *strip_prefix = func_data;
+  const char *result;
+  
+  if (strip_prefix)
+    {
+      char *tmp_path = g_strconcat (strip_prefix, path, NULL);
+      result = gettext (tmp_path);
+      if (result == tmp_path)
+	result = path;
+      g_free (tmp_path);
+    }
+  else
+    result = gettext (path);
+
+  return (char *)result;
+}
+
+
+GtkItemFactory *
+item_factory_new (GtkType container_type, const char *path,
+		  GtkAccelGroup *accel_group, const char *strip_prefix)
+{
+  GtkItemFactory *result = gtk_item_factory_new (container_type, path, accel_group);
+  gchar *strip_prefix_dup = g_strdup (g_strdup (strip_prefix));
+  
+  gtk_item_factory_set_translate_func (result, gftp_item_factory_translate,
+				       strip_prefix_dup, NULL);
+
+  if (strip_prefix_dup)
+    gtk_object_set_data_full (GTK_OBJECT (result), "gftp-strip-prefix",
+			      strip_prefix_dup, (GDestroyNotify)g_free);
+
+  return result;
+}
+
+
 void
 create_item_factory (GtkItemFactory * ifactory, guint n_entries,
 		     GtkItemFactoryEntry * entries, gpointer callback_data)
 {
-  GtkItemFactoryEntry dummy_item;
   int i;
+  const char *strip_prefix = gtk_object_get_data (GTK_OBJECT (ifactory), "gftp-strip-prefix");
+  int strip_prefix_len = 0;
+
+  if (strip_prefix)
+    strip_prefix_len = strlen (strip_prefix);
 
   for (i = 0; i < n_entries; i++)
     {
-      memcpy (&dummy_item, entries + i, sizeof (dummy_item));
-      if (dummy_item.item_type)
-	dummy_item.item_type = _(entries[i].item_type);
-      dummy_item.path = _(entries[i].path);
+      GtkItemFactoryEntry dummy_item = entries[i];
+      if (strip_prefix && strncmp (entries[i].path, strip_prefix, strip_prefix_len) == 0)
+	dummy_item.path += strip_prefix_len;
+      
       gtk_item_factory_create_item (ifactory, &dummy_item, callback_data, 1);
     }
 }
 
-
 GList *
 get_next_selection (GList * selection, GList ** list, int *curnum)
 {
@@ -625,7 +687,8 @@ add_history (GtkWidget * widget, GList ** history, unsigned int *histlen,
 int
 check_reconnect (gftp_window_data *wdata)
 {
-  return (wdata->request->cached && wdata->request->sockfd == NULL && 
+  return (wdata->request->cached && wdata->request->sockfd < 0 && 
+          !wdata->request->always_connected &&
 	  !ftp_connect (wdata, wdata->request, 0) ? -1 : 0);
 }
 
@@ -709,8 +772,8 @@ add_file_listbox (gftp_window_data * wdata, gftp_file * fle)
     gtk_clist_set_text (GTK_CLIST (wdata->listbox), clist_num, 1, fle->file);
 
   if (fle->attribs && (*fle->attribs == 'b' || *fle->attribs == 'c'))
-    tempstr = g_strdup_printf ("%d, %d", (int) fle->size >> 16,
-                               (int) fle->size & 0xFF);
+    tempstr = g_strdup_printf ("%d, %d", major (fle->size),
+                               minor (fle->size));
   else
     tempstr = insert_commas (fle->size, NULL, 0);
 
@@ -733,182 +796,298 @@ add_file_listbox (gftp_window_data * wdata, gftp_file * fle)
 }
 
 
-void
-MakeEditDialog (char *diagtxt, char *infotxt, char *deftext, int edit_shown,
-		int erase, char *checktext, 
-                char *oktxt, void (*okfunc) (), void *okptr,
-		char *canceltxt, void (*cancelfunc) (), void *cancelptr)
+#if GTK_MAJOR_VERSION == 1
+static void
+ok_dialog_response (GtkWidget * widget, gftp_dialog_data * ddata)
 {
-  gftp_dialog_data * d, * cancel_d;
-  GtkWidget * tempwid;
+  if (ddata->edit == NULL)
+    gtk_widget_destroy (ddata->dialog);
+ 
+  if (ddata->yesfunc != NULL)
+    ddata->yesfunc (ddata->yespointer, ddata);
 
-  d = g_malloc0 (sizeof (*d));
-  cancel_d = g_malloc0 (sizeof (*cancel_d));
-  cancel_d->data = cancelptr;
-  d->data = okptr;
-  cancel_d->all_buttons = d->all_buttons = NULL;
-  d->dialog = cancel_d->dialog = gtk_dialog_new ();
-  gtk_grab_add (d->dialog);
-  gtk_window_set_title (GTK_WINDOW (d->dialog), diagtxt);
-  gtk_window_set_wmclass (GTK_WINDOW(d->dialog), "edit", "gFTP");
-  gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (d->dialog)->vbox), 10);
-  gtk_container_border_width (GTK_CONTAINER
-			      (GTK_DIALOG (d->dialog)->action_area), 5);
-  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (d->dialog)->vbox), 5);
-  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (d->dialog)->action_area), 15);
-  gtk_box_set_homogeneous (GTK_BOX (GTK_DIALOG (d->dialog)->action_area), TRUE);
-  gtk_window_set_position (GTK_WINDOW (d->dialog), GTK_WIN_POS_MOUSE);
+  if (ddata->edit != NULL)
+    gtk_widget_destroy (ddata->dialog);
 
-  if (cancelfunc)
-    gtk_signal_connect (GTK_OBJECT (d->dialog), "delete_event",
-			GTK_SIGNAL_FUNC (cancelfunc), (gpointer) cancel_d);
-
-  gtk_signal_connect_object (GTK_OBJECT (d->dialog), "delete_event",
-			     GTK_SIGNAL_FUNC (destroy_dialog_data),
-			     (gpointer) cancel_d);
-  tempwid = gtk_label_new (infotxt);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->vbox), tempwid, TRUE,
-		      TRUE, 0);
-  gtk_widget_show (tempwid);
-
-  d->edit = cancel_d->edit = gtk_entry_new ();
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->vbox), d->edit, TRUE,
-		      TRUE, 0);
-  if (okfunc)
-    gtk_signal_connect (GTK_OBJECT (d->edit), "activate",
-			GTK_SIGNAL_FUNC (okfunc), (gpointer) d);
-  if (erase || !okfunc)
-    gtk_signal_connect_object (GTK_OBJECT (d->edit), "activate",
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       (gpointer) d->dialog);
-
-  gtk_widget_grab_focus (d->edit);
-  gtk_entry_set_visibility (GTK_ENTRY (d->edit), edit_shown);
-  if (deftext != NULL)
-    {
-      gtk_entry_set_text (GTK_ENTRY (d->edit), deftext);
-      gtk_entry_select_region (GTK_ENTRY (d->edit), 0, strlen (deftext));
-    }
-  gtk_widget_show (d->edit);
-
-  if (checktext)
-    {
-      d->checkbox = cancel_d->checkbox = 
-                                   gtk_check_button_new_with_label (checktext);
-      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->vbox), d->checkbox, 
-                          TRUE, TRUE, 0);
-      gtk_widget_show (d->checkbox);
-    }
-      
-  tempwid = gtk_button_new_with_label (oktxt);
-  GTK_WIDGET_SET_FLAGS (tempwid, GTK_CAN_DEFAULT);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->action_area), tempwid,
-		      TRUE, TRUE, 0);
-  if (okfunc)
-    gtk_signal_connect (GTK_OBJECT (tempwid), "clicked",
-			GTK_SIGNAL_FUNC (okfunc), (gpointer) d);
-  if (erase || !okfunc)
-    gtk_signal_connect_object (GTK_OBJECT (tempwid), "clicked",
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       (gpointer) d->dialog);
-  gtk_widget_grab_default (tempwid);
-  gtk_widget_show (tempwid);
-
-  tempwid = gtk_button_new_with_label (canceltxt);
-  GTK_WIDGET_SET_FLAGS (tempwid, GTK_CAN_DEFAULT);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d->dialog)->action_area), tempwid,
-		      TRUE, TRUE, 0);
-  if (cancelfunc)
-    gtk_signal_connect (GTK_OBJECT (tempwid), "clicked",
-			GTK_SIGNAL_FUNC (cancelfunc), (gpointer) cancel_d);
-  if (erase || !cancelfunc)
-    gtk_signal_connect_object (GTK_OBJECT (tempwid), "clicked",
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       (gpointer) d->dialog);
-  gtk_widget_show (tempwid);
-
-  d->all_buttons = g_list_append (d->all_buttons, d);
-  d->all_buttons = g_list_append (d->all_buttons, cancel_d);
-  cancel_d->all_buttons = d->all_buttons;
-  gtk_widget_show (d->dialog);
+  g_free (ddata);
 }
 
 
-void
-MakeYesNoDialog (char *diagtxt, char *infotxt, int erase, int num, ...)
+static void
+cancel_dialog_response (GtkWidget * widget, gftp_dialog_data * ddata)
 {
-  GtkWidget * text, * tempwid, * dialog;
-  typedef void (*func) ();
-  gftp_dialog_data * d;
-  char *tempstr;
-  va_list argp;
-  func myfunc;
-  GList * list;
-  void *ptr;
-  int i;
+  if (ddata->edit == NULL)
+    gtk_widget_destroy (ddata->dialog);
+ 
+  if (ddata->nofunc != NULL)
+    ddata->nofunc (ddata->nopointer, ddata);
 
+  if (ddata->edit != NULL)
+    gtk_widget_destroy (ddata->dialog);
+
+  g_free (ddata);
+}
+#else
+static void
+dialog_response (GtkWidget * widget, gint response, gftp_dialog_data * ddata)
+{
+  if (ddata->edit == NULL)
+    gtk_widget_destroy (widget);
+
+  switch (response)
+    {
+      case GTK_RESPONSE_YES:
+        if (ddata->yesfunc != NULL)
+          ddata->yesfunc (ddata->yespointer, ddata);
+        break;
+      default:
+        if (ddata->nofunc != NULL)
+          ddata->nofunc (ddata->nopointer, ddata);
+        break;
+    }
+
+  if (ddata->edit != NULL)
+    gtk_widget_destroy (widget);
+
+  g_free (ddata);
+}
+#endif
+
+
+void
+MakeEditDialog (char *diagtxt, char *infotxt, char *deftext, int passwd_item,
+		char *checktext, 
+                gftp_dialog_button okbutton, void (*okfunc) (), void *okptr,
+		void (*cancelfunc) (), void *cancelptr)
+{
+  GtkWidget * tempwid, * dialog;
+  gftp_dialog_data * ddata;
+  const gchar * yes_text;
+
+  ddata = g_malloc (sizeof (*ddata));
+  ddata->yesfunc = okfunc;
+  ddata->yespointer = okptr;
+  ddata->nofunc = cancelfunc;
+  ddata->nopointer = cancelptr;
+
+#if GTK_MAJOR_VERSION == 1
   dialog = gtk_dialog_new ();
-  gtk_grab_add (dialog);
   gtk_window_set_title (GTK_WINDOW (dialog), diagtxt);
-  gtk_window_set_wmclass (GTK_WINDOW(dialog), "yndiag", "gFTP");
-  gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), 10);
-  gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->action_area), 
-                              5);
-  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 5);
+  gtk_grab_add (dialog);
+  gtk_container_border_width (GTK_CONTAINER
+			      (GTK_DIALOG (dialog)->action_area), 5);
   gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->action_area), 15);
   gtk_box_set_homogeneous (GTK_BOX (GTK_DIALOG (dialog)->action_area), TRUE);
-  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
-
-  text = gtk_label_new (infotxt);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), text, TRUE, TRUE, 0);
-  gtk_widget_show (text);
-
-  list = NULL;
-  ptr = g_malloc0 (sizeof (gftp_dialog_data));
-  list = g_list_append (list, ptr);
-  va_start (argp, num);
-  for (i = 0; i < num; i++)
+#else
+  switch (okbutton)
     {
-      tempstr = va_arg (argp, char *);
-      myfunc = va_arg (argp, func); 
-      ptr = va_arg (argp, void *);
-      d = g_malloc (sizeof (gftp_dialog_data));
-      list = g_list_append (list, d);
-      d->dialog = dialog;
-      d->data = ptr;
-      d->all_buttons = list;
-
-      tempwid = gtk_button_new_with_label (tempstr);
-      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), tempwid,
-			  TRUE, TRUE, 0);
-      if (myfunc)
-	gtk_signal_connect (GTK_OBJECT (tempwid), "clicked",
-			    GTK_SIGNAL_FUNC (myfunc), (gpointer) d);
-      if (erase || !myfunc)
-	gtk_signal_connect_object (GTK_OBJECT (tempwid), "clicked",
-				   GTK_SIGNAL_FUNC (gtk_widget_destroy),
-				   (gpointer) d->dialog);
-      gtk_widget_show (tempwid);
+      case gftp_dialog_button_create:
+        yes_text = GTK_STOCK_ADD;
+        break;
+      case gftp_dialog_button_change:
+        yes_text = _("Change");
+        break;
+      case gftp_dialog_button_connect:
+        yes_text = _("Connect");
+        break;
+      case gftp_dialog_button_rename:
+        yes_text = _("Rename");
+        break;
+      default:
+        yes_text = GTK_STOCK_MISSING_IMAGE;
+        break;
     }
 
+  dialog = gtk_dialog_new_with_buttons (_(diagtxt), NULL, 0,
+                                        yes_text,
+                                        GTK_RESPONSE_YES,
+                                        GTK_STOCK_CANCEL,
+                                        GTK_RESPONSE_NO,
+                                        NULL);
+#endif
+  gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), 10);
+  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 5);
+  gtk_window_set_wmclass (GTK_WINDOW(dialog), "edit", "gFTP");
+  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
+  gtk_widget_realize (dialog);
+
+  if (gftp_icon != NULL)
+    {
+      gdk_window_set_icon (dialog->window, NULL, gftp_icon->pixmap,
+                           gftp_icon->bitmap);
+      gdk_window_set_icon_name (dialog->window, _("gFTP Icon"));
+    }
+
+  ddata->dialog = dialog;
+
+  tempwid = gtk_label_new (infotxt);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), tempwid, TRUE,
+		      TRUE, 0);
+  gtk_widget_show (tempwid);
+
+  ddata->edit = gtk_entry_new ();
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), ddata->edit, TRUE,
+		      TRUE, 0);
+  gtk_widget_grab_focus (ddata->edit);
+  gtk_entry_set_visibility (GTK_ENTRY (ddata->edit), passwd_item);
+
+  if (deftext != NULL)
+    {
+      gtk_entry_set_text (GTK_ENTRY (ddata->edit), deftext);
+      gtk_entry_select_region (GTK_ENTRY (ddata->edit), 0, strlen (deftext));
+    }
+  gtk_widget_show (ddata->edit);
+
+  if (checktext != NULL)
+    {
+      ddata->checkbox = gtk_check_button_new_with_label (checktext);
+      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
+                          ddata->checkbox, TRUE, TRUE, 0);
+      gtk_widget_show (ddata->checkbox);
+    }
+      
+#if GTK_MAJOR_VERSION == 1
+  switch (okbutton)
+    {
+      case gftp_dialog_button_create:
+        yes_text = _("Add");
+        break;
+      case gftp_dialog_button_change:
+        yes_text = _("Change");
+        break;
+      case gftp_dialog_button_connect:
+        yes_text = _("Connect");
+        break;
+      case gftp_dialog_button_rename:
+        yes_text = _("Rename");
+        break;
+      default:
+        yes_text = "";
+        break;
+    }
+
+  tempwid = gtk_button_new_with_label (yes_text);
+  GTK_WIDGET_SET_FLAGS (tempwid, GTK_CAN_DEFAULT);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), tempwid,
+		      TRUE, TRUE, 0);
+  gtk_signal_connect (GTK_OBJECT (tempwid), "clicked",
+                      GTK_SIGNAL_FUNC (ok_dialog_response),
+                      ddata);
+  gtk_widget_grab_default (tempwid);
+  gtk_widget_show (tempwid);
+
+  tempwid = gtk_button_new_with_label (_("Cancel"));
+  GTK_WIDGET_SET_FLAGS (tempwid, GTK_CAN_DEFAULT);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), tempwid,
+		      TRUE, TRUE, 0);
+  gtk_signal_connect (GTK_OBJECT (tempwid), "clicked",
+                      GTK_SIGNAL_FUNC (cancel_dialog_response),
+                      ddata);
+  gtk_widget_show (tempwid);
+#else
+  g_signal_connect (GTK_OBJECT (dialog), "response",
+                    G_CALLBACK (dialog_response), ddata);
+#endif
+
   gtk_widget_show (dialog);
 }
 
 
-static void
-destroy_dialog_data (gftp_dialog_data * data)
+void
+MakeYesNoDialog (char *diagtxt, char *infotxt, 
+                 void (*yesfunc) (), gpointer yespointer, 
+                 void (*nofunc) (), gpointer nopointer)
 {
-  GList * list;
+  GtkWidget * text, * dialog;
+  gftp_dialog_data * ddata;
+#if GTK_MAJOR_VERSION == 1
+  GtkWidget * tempwid;
+#endif
 
-  list = data->all_buttons;
-  while (list != NULL)
+  ddata = g_malloc (sizeof (*ddata));
+  ddata->yesfunc = yesfunc;
+  ddata->yespointer = yespointer;
+  ddata->nofunc = nofunc;
+  ddata->nopointer = nopointer;
+
+#if GTK_MAJOR_VERSION == 1
+  dialog = gtk_dialog_new ();
+  gtk_grab_add (dialog);
+  gtk_window_set_title (GTK_WINDOW (dialog), diagtxt);
+  gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->action_area), 
+                              5);
+  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->action_area), 15);
+  gtk_box_set_homogeneous (GTK_BOX (GTK_DIALOG (dialog)->action_area), TRUE);
+#else
+  dialog = gtk_dialog_new_with_buttons (_(diagtxt), NULL, 0,
+                                        GTK_STOCK_YES,
+                                        GTK_RESPONSE_YES,
+                                        GTK_STOCK_NO,
+                                        GTK_RESPONSE_NO,
+                                        NULL);
+#endif
+  gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), 10);
+  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 5);
+  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
+  gtk_window_set_wmclass (GTK_WINDOW(dialog), "yndiag", "gFTP");
+  gtk_widget_realize (dialog);
+
+  if (gftp_icon != NULL)
     {
-      if (list->data != data && list->data)
-        g_free (list->data);
-      list = list->next;
+      gdk_window_set_icon (dialog->window, NULL, gftp_icon->pixmap,
+                           gftp_icon->bitmap);
+      gdk_window_set_icon_name (dialog->window, _("gFTP Icon"));
     }
-  g_list_free (data->all_buttons);
-  g_free (data);
+
+  ddata->dialog = dialog;
+
+  text = gtk_label_new (infotxt);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), text, TRUE, TRUE, 0);
+  gtk_widget_show (text);
+
+#if GTK_MAJOR_VERSION == 1
+  tempwid = gtk_button_new_with_label (_("  Yes  "));
+  GTK_WIDGET_SET_FLAGS (tempwid, GTK_CAN_DEFAULT);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), tempwid,
+                      FALSE, FALSE, 0);
+  gtk_signal_connect (GTK_OBJECT (tempwid), "clicked",
+                      GTK_SIGNAL_FUNC (ok_dialog_response),
+                      ddata);
+  gtk_widget_grab_default (tempwid);
+  gtk_widget_show (tempwid);
+
+  tempwid = gtk_button_new_with_label (_("  No  "));
+  GTK_WIDGET_SET_FLAGS (tempwid, GTK_CAN_DEFAULT);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), tempwid,
+                      FALSE, FALSE, 0);
+  gtk_signal_connect (GTK_OBJECT (tempwid), "clicked",
+                      GTK_SIGNAL_FUNC (cancel_dialog_response),
+                      ddata);
+  gtk_widget_show (tempwid);
+
+#else
+  g_signal_connect (GTK_OBJECT (dialog), "response",
+                    G_CALLBACK (dialog_response), ddata);
+#endif
+
+  gtk_widget_show (dialog);
+}
+
+
+static gint
+delete_event (GtkWidget * widget, GdkEvent * event, gpointer data)
+{
+  return (TRUE);
+}
+
+
+static void
+trans_stop_button (GtkWidget * widget, gpointer data)
+{
+  gftp_transfer * transfer;
+
+  transfer = data;
+  pthread_kill (((gftp_window_data *) transfer->fromwdata)->tid, SIGINT);
 }
 
 
@@ -982,7 +1161,14 @@ generic_thread (void * (*func) (void *), gftp_window_data * wdata)
       pthread_create (&wdata->tid, NULL, func, wdata);
 
       while (wdata->request->stopable)
-        g_main_iteration (TRUE);
+        {
+          GDK_THREADS_LEAVE ();
+#if GTK_MAJOR_VERSION == 1
+          g_main_iteration (TRUE);
+#else
+          g_main_context_iteration (NULL, TRUE);
+#endif
+        }
 
       pthread_join (wdata->tid, &ret);
       gtk_widget_set_sensitive (stop_btn, 0);
@@ -1018,22 +1204,6 @@ progress_timeout (gpointer data)
 }
 
 
-static gint
-delete_event (GtkWidget * widget, GdkEvent * event, gpointer data)
-{
-  return (TRUE);
-}
-
-
-static void
-trans_stop_button (GtkWidget * widget, gpointer data)
-{
-  gftp_transfer * transfer;
-
-  transfer = data;
-  pthread_kill (((gftp_window_data *) transfer->fromwdata)->tid, SIGINT);
-}
-
 void
 display_cached_logs (void)
 {
@@ -1056,9 +1226,20 @@ display_cached_logs (void)
   pthread_mutex_unlock (&log_mutex);
 }
 
-#if !defined (HAVE_GETADDRINFO) || !defined (HAVE_GAI_STRERROR)
 
-static pthread_mutex_t netfunclock = PTHREAD_MUTEX_INITIALIZER;
+RETSIGTYPE 
+signal_handler (int signo)
+{
+  signal (signo, signal_handler);
+
+  if (use_jmp_environment)
+    siglongjmp (jmp_environment, signo == SIGINT ? 1 : 2);
+  else if (signo == SIGINT)
+    exit (1);
+}
+
+
+#if !defined (HAVE_GETADDRINFO) || !defined (HAVE_GAI_STRERROR)
 
 struct hostent *
 r_gethostbyname (const char *name, struct hostent *result_buf, int *h_errnop)
@@ -1080,6 +1261,7 @@ r_gethostbyname (const char *name, struct hostent *result_buf, int *h_errnop)
   return (hent);
 }
 
+#endif /* !HAVE_GETADDRINFO */
 
 struct servent *
 r_getservbyname (const char *name, const char *proto,
@@ -1102,5 +1284,3 @@ r_getservbyname (const char *name, const char *proto,
   return (sent);
 }
 
-#endif /* !HAVE_GETADDRINFO */
-
